# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: qm/grpc/v2/qm_api.proto
# Protobuf Python Version: 5.28.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    28,
    1,
    '',
    'qm/grpc/v2/qm_api.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from qm.grpc.qm.grpc.v2 import common_types_pb2 as qm_dot_grpc_dot_v2_dot_common__types__pb2
from qm.grpc.qm.grpc.v2 import qmm_api_pb2 as qm_dot_grpc_dot_v2_dot_qmm__api__pb2
from qm.grpc.qm.pb import compiler_pb2 as qm_dot_pb_dot_compiler__pb2
from qm.grpc.qm.pb import frontend_pb2 as qm_dot_pb_dot_frontend__pb2
from qm.grpc.qm.pb import inc_qua_pb2 as qm_dot_pb_dot_inc__qua__pb2
from qm.grpc.qm.pb import inc_qua_config_pb2 as qm_dot_pb_dot_inc__qua__config__pb2
from qm.grpc.qm.pb import qm_manager_pb2 as qm_dot_pb_dot_qm__manager__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x17qm/grpc/v2/qm_api.proto\x12\nqm.grpc.v2\x1a\x1dqm/grpc/v2/common_types.proto\x1a\x18qm/grpc/v2/qmm_api.proto\x1a\x14qm/pb/compiler.proto\x1a\x14qm/pb/frontend.proto\x1a\x13qm/pb/inc_qua.proto\x1a\x1aqm/pb/inc_qua_config.proto\x1a\x16qm/pb/qm_manager.proto\"7\n\x19QmServiceGetConfigRequest\x12\x1a\n\x12quantum_machine_id\x18\x01 \x01(\t\"`\n\x13UpdateConfigRequest\x12\x1a\n\x12quantum_machine_id\x18\x01 \x01(\t\x12-\n\x06\x63onfig\x18\x02 \x01(\x0b\x32\x1d.qm.grpc.qua_config.QuaConfig\"\xe2\x02\n\x14UpdateConfigResponse\x12O\n\x07success\x18\x01 \x01(\x0b\x32<.qm.grpc.v2.UpdateConfigResponse.UpdateConfigResponseSuccessH\x00\x12K\n\x05\x65rror\x18\x02 \x01(\x0b\x32:.qm.grpc.v2.UpdateConfigResponse.UpdateConfigResponseErrorH\x00\x1a\x1d\n\x1bUpdateConfigResponseSuccess\x1a{\n\x19UpdateConfigResponseError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\t\x12M\n\x18\x63onfig_validation_errors\x18\x02 \x03(\x0b\x32+.qm.grpc.qm_manager.ConfigValidationMessageB\x10\n\x0eresponse_oneof\"3\n\x15QmServiceCloseRequest\x12\x1a\n\x12quantum_machine_id\x18\x01 \x01(\t\"\xa1\x02\n\x16QmServiceCloseResponse\x12S\n\x07success\x18\x01 \x01(\x0b\x32@.qm.grpc.v2.QmServiceCloseResponse.QmServiceCloseResponseSuccessH\x00\x12O\n\x05\x65rror\x18\x02 \x01(\x0b\x32>.qm.grpc.v2.QmServiceCloseResponse.QmServiceCloseResponseErrorH\x00\x1a\x1f\n\x1dQmServiceCloseResponseSuccess\x1a.\n\x1bQmServiceCloseResponseError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\tB\x10\n\x0eresponse_oneof\"\xaf\x02\n\x18QmServiceSimulateRequest\x12\x1a\n\x12quantum_machine_id\x18\x01 \x01(\t\x12\x33\n\x12high_level_program\x18\x02 \x01(\x0b\x32\x17.qm.grpc.qua.QuaProgram\x12=\n\x08simulate\x18\x03 \x01(\x0b\x32+.qm.grpc.frontend.ExecutionRequest.Simulate\x12\x44\n\x16\x63ontroller_connections\x18\x04 \x03(\x0b\x32$.qm.grpc.frontend.InterOpxConnection\x12\x32\n\x06\x63onfig\x18\x05 \x01(\x0b\x32\x1d.qm.grpc.qua_config.QuaConfigH\x00\x88\x01\x01\x42\t\n\x07_config\"\x8d\x01\n\x19QmServiceSimulateResponse\x12\x30\n\x07success\x18\x01 \x01(\x0b\x32\x1d.qm.grpc.v2.SimulationSuccessH\x00\x12,\n\x05\x65rror\x18\x02 \x01(\x0b\x32\x1b.qm.grpc.v2.SimulationErrorH\x00\x42\x10\n\x0eresponse_oneof\"\x9e\x01\n\x18QmServiceJobsQueryParams\x12\x1a\n\x12quantum_machine_id\x18\x01 \x01(\t\x12\x0f\n\x07job_ids\x18\x02 \x03(\t\x12\x10\n\x08user_ids\x18\x03 \x03(\t\x12\x13\n\x0b\x64\x65scription\x18\x04 \x01(\t\x12.\n\x06status\x18\x05 \x03(\x0e\x32\x1e.qm.grpc.v2.JobExecutionStatus\"E\n\x17QmServiceGetJobsRequest\x12*\n\x05query\x18\x01 \x01(\x0b\x32\x1b.qm.grpc.v2.JobsQueryParams\"?\n\x11RemoveJobsRequest\x12*\n\x05query\x18\x01 \x01(\x0b\x32\x1b.qm.grpc.v2.JobsQueryParams\"\x9e\x02\n\x12RemoveJobsResponse\x12K\n\x07success\x18\x01 \x01(\x0b\x32\x38.qm.grpc.v2.RemoveJobsResponse.RemoveJobsResponseSuccessH\x00\x12G\n\x05\x65rror\x18\x02 \x01(\x0b\x32\x36.qm.grpc.v2.RemoveJobsResponse.RemoveJobsResponseErrorH\x00\x1a\x34\n\x19RemoveJobsResponseSuccess\x12\x17\n\x0fremoved_job_ids\x18\x01 \x03(\t\x1a*\n\x17RemoveJobsResponseError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\tB\x10\n\x0eresponse_oneof\"\x8c\x01\n\x1aQmServiceGetConfigResponse\x12/\n\x07success\x18\x01 \x01(\x0b\x32\x1c.qm.grpc.v2.GetConfigSuccessH\x00\x12+\n\x05\x65rror\x18\x02 \x01(\x0b\x32\x1a.qm.grpc.v2.GetConfigErrorH\x00\x42\x10\n\x0eresponse_oneof\"A\n\x10GetConfigSuccess\x12-\n\x06\x63onfig\x18\x01 \x01(\x0b\x32\x1d.qm.grpc.qua_config.QuaConfig\"!\n\x0eGetConfigError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\t\"\xa9\x01\n\x17QmServiceCompileRequest\x12\x1a\n\x12quantum_machine_id\x18\x01 \x01(\t\x12\x33\n\x12high_level_program\x18\x02 \x01(\x0b\x32\x17.qm.grpc.qua.QuaProgram\x12\x32\n\x06\x63onfig\x18\x03 \x01(\x0b\x32\x1d.qm.grpc.qua_config.QuaConfigH\x00\x88\x01\x01\x42\t\n\x07_config\"\xae\x03\n\x0f\x43ompileResponse\x12\x41\n\x07success\x18\x01 \x01(\x0b\x32..qm.grpc.v2.CompileResponse.CompilationSuccessH\x00\x12=\n\x05\x65rror\x18\x02 \x01(\x0b\x32,.qm.grpc.v2.CompileResponse.CompilationErrorH\x00\x1a]\n\x12\x43ompilationSuccess\x12\x12\n\nprogram_id\x18\x01 \x01(\t\x12\x33\n\x08messages\x18\x02 \x03(\x0b\x32!.qm.grpc.compiler.CompilerMessage\x1a\xa7\x01\n\x10\x43ompilationError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\t\x12\x33\n\x08messages\x18\x02 \x03(\x0b\x32!.qm.grpc.compiler.CompilerMessage\x12M\n\x18\x63onfig_validation_errors\x18\x03 \x03(\x0b\x32+.qm.grpc.qm_manager.ConfigValidationMessageB\x10\n\x0eresponse_oneof\"\xe5\x01\n\x1aQmServiceAddToQueueRequest\x12\x1a\n\x12quantum_machine_id\x18\x01 \x01(\t\x12\x33\n\x12high_level_program\x18\x02 \x01(\x0b\x32\x17.qm.grpc.qua.QuaProgram\x12\x37\n\x0equeue_position\x18\x03 \x01(\x0b\x32\x1f.qm.grpc.frontend.QueuePosition\x12\x32\n\x06\x63onfig\x18\x04 \x01(\x0b\x32\x1d.qm.grpc.qua_config.QuaConfigH\x00\x88\x01\x01\x42\t\n\x07_config\"\xcf\x03\n\x12\x41\x64\x64ToQueueResponse\x12K\n\x07success\x18\x01 \x01(\x0b\x32\x38.qm.grpc.v2.AddToQueueResponse.AddToQueueResponseSuccessH\x00\x12G\n\x05\x65rror\x18\x02 \x01(\x0b\x32\x36.qm.grpc.v2.AddToQueueResponse.AddToQueueResponseErrorH\x00\x1a`\n\x19\x41\x64\x64ToQueueResponseSuccess\x12\x0e\n\x06job_id\x18\x01 \x01(\t\x12\x33\n\x08messages\x18\x02 \x03(\x0b\x32!.qm.grpc.compiler.CompilerMessage\x1a\xae\x01\n\x17\x41\x64\x64ToQueueResponseError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\t\x12\x33\n\x08messages\x18\x02 \x03(\x0b\x32!.qm.grpc.compiler.CompilerMessage\x12M\n\x18\x63onfig_validation_errors\x18\x03 \x03(\x0b\x32+.qm.grpc.qm_manager.ConfigValidationMessageB\x10\n\x0eresponse_oneof\"T\n\"QmServiceAddCompiledToQueueRequest\x12\x1a\n\x12quantum_machine_id\x18\x01 \x01(\t\x12\x12\n\nprogram_id\x18\x02 \x01(\t\"\xcd\x02\n\x1a\x41\x64\x64\x43ompiledToQueueResponse\x12[\n\x07success\x18\x01 \x01(\x0b\x32H.qm.grpc.v2.AddCompiledToQueueResponse.AddCompiledToQueueResponseSuccessH\x00\x12W\n\x05\x65rror\x18\x02 \x01(\x0b\x32\x46.qm.grpc.v2.AddCompiledToQueueResponse.AddCompiledToQueueResponseErrorH\x00\x1a\x33\n!AddCompiledToQueueResponseSuccess\x12\x0e\n\x06job_id\x18\x01 \x01(\t\x1a\x32\n\x1f\x41\x64\x64\x43ompiledToQueueResponseError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\tB\x10\n\x0eresponse_oneof\"\x86\x01\n\x18QmServiceGetJobsResponse\x12-\n\x07success\x18\x01 \x01(\x0b\x32\x1a.qm.grpc.v2.GetJobsSuccessH\x00\x12)\n\x05\x65rror\x18\x02 \x01(\x0b\x32\x18.qm.grpc.v2.GetJobsErrorH\x00\x42\x10\n\x0eresponse_oneof\"8\n\x1aResetDigitalFiltersRequest\x12\x1a\n\x12quantum_machine_id\x18\x01 \x01(\t\"\xc4\x02\n\x1bResetDigitalFiltersResponse\x12]\n\x07success\x18\x01 \x01(\x0b\x32J.qm.grpc.v2.ResetDigitalFiltersResponse.ResetDigitalFiltersResponseSuccessH\x00\x12Y\n\x05\x65rror\x18\x02 \x01(\x0b\x32H.qm.grpc.v2.ResetDigitalFiltersResponse.ResetDigitalFiltersResponseErrorH\x00\x1a$\n\"ResetDigitalFiltersResponseSuccess\x1a\x33\n ResetDigitalFiltersResponseError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\tB\x10\n\x0eresponse_oneof2\x93\x07\n\tQmService\x12S\n\x0cUpdateConfig\x12\x1f.qm.grpc.v2.UpdateConfigRequest\x1a .qm.grpc.v2.UpdateConfigResponse\"\x00\x12\\\n\tGetConfig\x12%.qm.grpc.v2.QmServiceGetConfigRequest\x1a&.qm.grpc.v2.QmServiceGetConfigResponse\"\x00\x12M\n\x07\x43ompile\x12#.qm.grpc.v2.QmServiceCompileRequest\x1a\x1b.qm.grpc.v2.CompileResponse\"\x00\x12P\n\x05\x43lose\x12!.qm.grpc.v2.QmServiceCloseRequest\x1a\".qm.grpc.v2.QmServiceCloseResponse\"\x00\x12V\n\nAddToQueue\x12&.qm.grpc.v2.QmServiceAddToQueueRequest\x1a\x1e.qm.grpc.v2.AddToQueueResponse\"\x00\x12n\n\x12\x41\x64\x64\x43ompiledToQueue\x12..qm.grpc.v2.QmServiceAddCompiledToQueueRequest\x1a&.qm.grpc.v2.AddCompiledToQueueResponse\"\x00\x12V\n\x07GetJobs\x12#.qm.grpc.v2.QmServiceGetJobsRequest\x1a$.qm.grpc.v2.QmServiceGetJobsResponse\"\x00\x12M\n\nRemoveJobs\x12\x1d.qm.grpc.v2.RemoveJobsRequest\x1a\x1e.qm.grpc.v2.RemoveJobsResponse\"\x00\x12Y\n\x08Simulate\x12$.qm.grpc.v2.QmServiceSimulateRequest\x1a%.qm.grpc.v2.QmServiceSimulateResponse\"\x00\x12h\n\x13ResetDigitalFilters\x12&.qm.grpc.v2.ResetDigitalFiltersRequest\x1a\'.qm.grpc.v2.ResetDigitalFiltersResponse\"\x00\x42\x1a\n\x10qm.client.api.v2P\x01\xa2\x02\x03HLWb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qm.grpc.v2.qm_api_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\020qm.client.api.v2P\001\242\002\003HLW'
  _globals['_QMSERVICEGETCONFIGREQUEST']._serialized_start=213
  _globals['_QMSERVICEGETCONFIGREQUEST']._serialized_end=268
  _globals['_UPDATECONFIGREQUEST']._serialized_start=270
  _globals['_UPDATECONFIGREQUEST']._serialized_end=366
  _globals['_UPDATECONFIGRESPONSE']._serialized_start=369
  _globals['_UPDATECONFIGRESPONSE']._serialized_end=723
  _globals['_UPDATECONFIGRESPONSE_UPDATECONFIGRESPONSESUCCESS']._serialized_start=551
  _globals['_UPDATECONFIGRESPONSE_UPDATECONFIGRESPONSESUCCESS']._serialized_end=580
  _globals['_UPDATECONFIGRESPONSE_UPDATECONFIGRESPONSEERROR']._serialized_start=582
  _globals['_UPDATECONFIGRESPONSE_UPDATECONFIGRESPONSEERROR']._serialized_end=705
  _globals['_QMSERVICECLOSEREQUEST']._serialized_start=725
  _globals['_QMSERVICECLOSEREQUEST']._serialized_end=776
  _globals['_QMSERVICECLOSERESPONSE']._serialized_start=779
  _globals['_QMSERVICECLOSERESPONSE']._serialized_end=1068
  _globals['_QMSERVICECLOSERESPONSE_QMSERVICECLOSERESPONSESUCCESS']._serialized_start=971
  _globals['_QMSERVICECLOSERESPONSE_QMSERVICECLOSERESPONSESUCCESS']._serialized_end=1002
  _globals['_QMSERVICECLOSERESPONSE_QMSERVICECLOSERESPONSEERROR']._serialized_start=1004
  _globals['_QMSERVICECLOSERESPONSE_QMSERVICECLOSERESPONSEERROR']._serialized_end=1050
  _globals['_QMSERVICESIMULATEREQUEST']._serialized_start=1071
  _globals['_QMSERVICESIMULATEREQUEST']._serialized_end=1374
  _globals['_QMSERVICESIMULATERESPONSE']._serialized_start=1377
  _globals['_QMSERVICESIMULATERESPONSE']._serialized_end=1518
  _globals['_QMSERVICEJOBSQUERYPARAMS']._serialized_start=1521
  _globals['_QMSERVICEJOBSQUERYPARAMS']._serialized_end=1679
  _globals['_QMSERVICEGETJOBSREQUEST']._serialized_start=1681
  _globals['_QMSERVICEGETJOBSREQUEST']._serialized_end=1750
  _globals['_REMOVEJOBSREQUEST']._serialized_start=1752
  _globals['_REMOVEJOBSREQUEST']._serialized_end=1815
  _globals['_REMOVEJOBSRESPONSE']._serialized_start=1818
  _globals['_REMOVEJOBSRESPONSE']._serialized_end=2104
  _globals['_REMOVEJOBSRESPONSE_REMOVEJOBSRESPONSESUCCESS']._serialized_start=1990
  _globals['_REMOVEJOBSRESPONSE_REMOVEJOBSRESPONSESUCCESS']._serialized_end=2042
  _globals['_REMOVEJOBSRESPONSE_REMOVEJOBSRESPONSEERROR']._serialized_start=2044
  _globals['_REMOVEJOBSRESPONSE_REMOVEJOBSRESPONSEERROR']._serialized_end=2086
  _globals['_QMSERVICEGETCONFIGRESPONSE']._serialized_start=2107
  _globals['_QMSERVICEGETCONFIGRESPONSE']._serialized_end=2247
  _globals['_GETCONFIGSUCCESS']._serialized_start=2249
  _globals['_GETCONFIGSUCCESS']._serialized_end=2314
  _globals['_GETCONFIGERROR']._serialized_start=2316
  _globals['_GETCONFIGERROR']._serialized_end=2349
  _globals['_QMSERVICECOMPILEREQUEST']._serialized_start=2352
  _globals['_QMSERVICECOMPILEREQUEST']._serialized_end=2521
  _globals['_COMPILERESPONSE']._serialized_start=2524
  _globals['_COMPILERESPONSE']._serialized_end=2954
  _globals['_COMPILERESPONSE_COMPILATIONSUCCESS']._serialized_start=2673
  _globals['_COMPILERESPONSE_COMPILATIONSUCCESS']._serialized_end=2766
  _globals['_COMPILERESPONSE_COMPILATIONERROR']._serialized_start=2769
  _globals['_COMPILERESPONSE_COMPILATIONERROR']._serialized_end=2936
  _globals['_QMSERVICEADDTOQUEUEREQUEST']._serialized_start=2957
  _globals['_QMSERVICEADDTOQUEUEREQUEST']._serialized_end=3186
  _globals['_ADDTOQUEUERESPONSE']._serialized_start=3189
  _globals['_ADDTOQUEUERESPONSE']._serialized_end=3652
  _globals['_ADDTOQUEUERESPONSE_ADDTOQUEUERESPONSESUCCESS']._serialized_start=3361
  _globals['_ADDTOQUEUERESPONSE_ADDTOQUEUERESPONSESUCCESS']._serialized_end=3457
  _globals['_ADDTOQUEUERESPONSE_ADDTOQUEUERESPONSEERROR']._serialized_start=3460
  _globals['_ADDTOQUEUERESPONSE_ADDTOQUEUERESPONSEERROR']._serialized_end=3634
  _globals['_QMSERVICEADDCOMPILEDTOQUEUEREQUEST']._serialized_start=3654
  _globals['_QMSERVICEADDCOMPILEDTOQUEUEREQUEST']._serialized_end=3738
  _globals['_ADDCOMPILEDTOQUEUERESPONSE']._serialized_start=3741
  _globals['_ADDCOMPILEDTOQUEUERESPONSE']._serialized_end=4074
  _globals['_ADDCOMPILEDTOQUEUERESPONSE_ADDCOMPILEDTOQUEUERESPONSESUCCESS']._serialized_start=3953
  _globals['_ADDCOMPILEDTOQUEUERESPONSE_ADDCOMPILEDTOQUEUERESPONSESUCCESS']._serialized_end=4004
  _globals['_ADDCOMPILEDTOQUEUERESPONSE_ADDCOMPILEDTOQUEUERESPONSEERROR']._serialized_start=4006
  _globals['_ADDCOMPILEDTOQUEUERESPONSE_ADDCOMPILEDTOQUEUERESPONSEERROR']._serialized_end=4056
  _globals['_QMSERVICEGETJOBSRESPONSE']._serialized_start=4077
  _globals['_QMSERVICEGETJOBSRESPONSE']._serialized_end=4211
  _globals['_RESETDIGITALFILTERSREQUEST']._serialized_start=4213
  _globals['_RESETDIGITALFILTERSREQUEST']._serialized_end=4269
  _globals['_RESETDIGITALFILTERSRESPONSE']._serialized_start=4272
  _globals['_RESETDIGITALFILTERSRESPONSE']._serialized_end=4596
  _globals['_RESETDIGITALFILTERSRESPONSE_RESETDIGITALFILTERSRESPONSESUCCESS']._serialized_start=4489
  _globals['_RESETDIGITALFILTERSRESPONSE_RESETDIGITALFILTERSRESPONSESUCCESS']._serialized_end=4525
  _globals['_RESETDIGITALFILTERSRESPONSE_RESETDIGITALFILTERSRESPONSEERROR']._serialized_start=4527
  _globals['_RESETDIGITALFILTERSRESPONSE_RESETDIGITALFILTERSRESPONSEERROR']._serialized_end=4578
  _globals['_QMSERVICE']._serialized_start=4599
  _globals['_QMSERVICE']._serialized_end=5514
# @@protoc_insertion_point(module_scope)



from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from typing import ClassVar as _ClassVar

DESCRIPTOR: _descriptor.FileDescriptor

class JobManagerErrorTypes(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    JobManagerUnspecifiedError: _ClassVar[JobManagerErrorTypes]
    MissingJobError: _ClassVar[JobManagerErrorTypes]
    InvalidJobExecutionStatusError: _ClassVar[JobManagerErrorTypes]
    InvalidOperationOnSimulatorJobError: _ClassVar[JobManagerErrorTypes]
    InvalidOperationOnRealJobError: _ClassVar[JobManagerErrorTypes]
    JobOperationSpecificError: _ClassVar[JobManagerErrorTypes]
    ConfigQueryError: _ClassVar[JobManagerErrorTypes]
    UnknownInputStreamError: _ClassVar[JobManagerErrorTypes]

class JobOperationSpecificErrorTypes(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    JobOperationUnspecifiedError: _ClassVar[JobOperationSpecificErrorTypes]
    SingleInputElementError: _ClassVar[JobOperationSpecificErrorTypes]
    InvalidCorrectionMatrixError: _ClassVar[JobOperationSpecificErrorTypes]
    ElementWithoutIntermediateFrequencyError: _ClassVar[JobOperationSpecificErrorTypes]
    InvalidDigitalInputThresholdError: _ClassVar[JobOperationSpecificErrorTypes]
    InvalidDigitalInputDeadtimeError: _ClassVar[JobOperationSpecificErrorTypes]
    InvalidDigitalInputPolarityError: _ClassVar[JobOperationSpecificErrorTypes]

class ConfigQueryErrorTypes(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    ConfigQueryUnspecifiedError: _ClassVar[ConfigQueryErrorTypes]
    MissingControllerError: _ClassVar[ConfigQueryErrorTypes]
    MissingElementError: _ClassVar[ConfigQueryErrorTypes]
    MissingDigitalInputError: _ClassVar[ConfigQueryErrorTypes]
JobManagerUnspecifiedError: JobManagerErrorTypes
MissingJobError: JobManagerErrorTypes
InvalidJobExecutionStatusError: JobManagerErrorTypes
InvalidOperationOnSimulatorJobError: JobManagerErrorTypes
InvalidOperationOnRealJobError: JobManagerErrorTypes
JobOperationSpecificError: JobManagerErrorTypes
ConfigQueryError: JobManagerErrorTypes
UnknownInputStreamError: JobManagerErrorTypes
JobOperationUnspecifiedError: JobOperationSpecificErrorTypes
SingleInputElementError: JobOperationSpecificErrorTypes
InvalidCorrectionMatrixError: JobOperationSpecificErrorTypes
ElementWithoutIntermediateFrequencyError: JobOperationSpecificErrorTypes
InvalidDigitalInputThresholdError: JobOperationSpecificErrorTypes
InvalidDigitalInputDeadtimeError: JobOperationSpecificErrorTypes
InvalidDigitalInputPolarityError: JobOperationSpecificErrorTypes
ConfigQueryUnspecifiedError: ConfigQueryErrorTypes
MissingControllerError: ConfigQueryErrorTypes
MissingElementError: ConfigQueryErrorTypes
MissingDigitalInputError: ConfigQueryErrorTypes



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: qm/pb/job_results.proto
# Protobuf Python Version: 5.28.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    28,
    1,
    '',
    'qm/pb/job_results.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from qm.grpc.qm.pb import general_messages_pb2 as qm_dot_pb_dot_general__messages__pb2
from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x17qm/pb/job_results.proto\x12\x18qm.grpc.results_analyser\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1cqm/pb/general_messages.proto\x1a\x1cgoogle/protobuf/struct.proto\"*\n\x19GetJobResultSchemaRequest\x12\r\n\x05jobId\x18\x01 \x01(\t\"\xc9\x01\n\x1aGetJobResultSchemaResponse\x12H\n\x05items\x18\x01 \x03(\x0b\x32\x39.qm.grpc.results_analyser.GetJobResultSchemaResponse.Item\x1a\x61\n\x04Item\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0bsimpleDType\x18\x02 \x01(\t\x12\x10\n\x08isSingle\x18\x03 \x01(\x08\x12\x15\n\rexpectedCount\x18\x04 \x01(\x05\x12\r\n\x05shape\x18\x05 \x03(\x05\"#\n\x12GetJobStateRequest\x12\r\n\x05jobId\x18\x01 \x01(\t\"H\n\x13GetJobStateResponse\x12\x0c\n\x04\x64one\x18\x01 \x01(\x08\x12\x0e\n\x06\x63losed\x18\x02 \x01(\x08\x12\x13\n\x0bhasDataloss\x18\x03 \x01(\x08\"W\n\x1eGetJobNamedResultHeaderRequest\x12\r\n\x05jobId\x18\x01 \x01(\t\x12\x12\n\noutputName\x18\x02 \x01(\t\x12\x12\n\nflatFormat\x18\x03 \x01(\x08\"\xde\x01\n\x1fGetJobNamedResultHeaderResponse\x12\x10\n\x08isSingle\x18\x01 \x01(\x08\x12\x12\n\ncountSoFar\x18\x02 \x01(\x05\x12\x13\n\x0bsimpleDType\x18\x03 \x01(\t\x12\x10\n\x04\x64one\x18\x04 \x01(\x08\x42\x02\x18\x01\x12\x12\n\x06\x63losed\x18\x05 \x01(\x08\x42\x02\x18\x01\x12\x13\n\x0bhasDataloss\x18\x06 \x01(\x08\x12\r\n\x05shape\x18\x07 \x03(\x05\x12\x36\n\x12hasExecutionErrors\x18\x08 \x01(\x0b\x32\x1a.google.protobuf.BoolValue\"\x91\x01\n\x18GetJobNamedResultRequest\x12\r\n\x05jobId\x18\x01 \x01(\t\x12\x12\n\noutputName\x18\x02 \x01(\t\x12\x12\n\x06offset\x18\x03 \x01(\x05\x42\x02\x18\x01\x12\r\n\x05limit\x18\x04 \x01(\x05\x12/\n\nlongOffset\x18\x05 \x01(\x0b\x32\x1b.google.protobuf.Int64Value\"?\n\x19GetJobNamedResultResponse\x12\x14\n\x0c\x63ountOfItems\x18\x01 \x01(\x05\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\x0c\"$\n\x13GetJobErrorsRequest\x12\r\n\x05jobId\x18\x01 \x01(\t\"\xa9\x02\n\x14GetJobErrorsResponse\x12\x44\n\x06\x65rrors\x18\x01 \x03(\x0b\x32\x34.qm.grpc.results_analyser.GetJobErrorsResponse.Error\x12\r\n\x05jobId\x18\x02 \x01(\t\x1a\x89\x01\n\x05\x45rror\x12\x11\n\terrorCode\x18\x01 \x01(\x05\x12\\\n\rerrorSeverity\x18\x02 \x01(\x0e\x32\x45.qm.grpc.results_analyser.GetJobErrorsResponse.ExecutionErrorSeverity\x12\x0f\n\x07message\x18\x03 \x01(\t\"0\n\x16\x45xecutionErrorSeverity\x12\t\n\x05\x45RROR\x10\x00\x12\x0b\n\x07WARNING\x10\x01\"\\\n\x1aPullAnalysedResultsRequest\x12\x13\n\x0bjobFilePath\x18\x01 \x01(\t\x12\x10\n\x08metadata\x18\x02 \x01(\t\x12\x17\n\x0f\x63ontainsVersion\x18\x03 \x01(\x08\"\xe8\x01\n\x17\x41nalysedResultsResponse\x12\x0f\n\x07version\x18\x01 \x01(\t\x12;\n\nicpResults\x18\x0b \x03(\x0b\x32\'.qm.grpc.results_analyser.IcpResultData\x12\x41\n\rstreamResults\x18\x0c \x03(\x0b\x32*.qm.grpc.results_analyser.StreamResultData\x12\x36\n\x06\x65rrors\x18\r \x03(\x0b\x32&.qm.grpc.general_messages.ErrorMessageJ\x04\x08\x02\x10\x0b\"<\n\x17PullResultOutputRequest\x12\r\n\x05jobId\x18\x01 \x01(\t\x12\x12\n\noutputName\x18\x02 \x01(\t\"\xca\x04\n\x18PullResultOutputResponse\x12G\n\x04\x64\x61ta\x18\x01 \x01(\x0b\x32\x37.qm.grpc.results_analyser.PullResultOutputResponse.DataH\x00\x12K\n\x06header\x18\x02 \x01(\x0b\x32\x39.qm.grpc.results_analyser.PullResultOutputResponse.HeaderH\x00\x12\x45\n\x03npz\x18\x03 \x01(\x0b\x32\x36.qm.grpc.results_analyser.PullResultOutputResponse.NpzH\x00\x12\x45\n\x03npy\x18\x04 \x01(\x0b\x32\x36.qm.grpc.results_analyser.PullResultOutputResponse.NpyH\x00\x1a\x87\x01\n\x06Header\x12\x0f\n\x07version\x18\x01 \x01(\t\x12\x12\n\noutputName\x18\x02 \x01(\t\x12\x10\n\x08\x64\x61taLoss\x18\x03 \x01(\x08\x12\x36\n\x06\x65rrors\x18\x04 \x03(\x0b\x32&.qm.grpc.general_messages.ErrorMessage\x12\x0e\n\x06single\x18\x05 \x01(\x08\x1a/\n\x04\x44\x61ta\x12\'\n\x07results\x18\x01 \x01(\x0b\x32\x16.google.protobuf.Value\x1a\x12\n\x03Npz\x12\x0b\n\x03npz\x18\x01 \x01(\x0c\x1a/\n\x03Npy\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0b\n\x03npy\x18\x02 \x01(\x0c\x12\r\n\x05\x63ount\x18\x03 \x01(\x03\x42\n\n\x08response\"\x1d\n\rBinaryWrapper\x12\x0c\n\x04\x64\x61ta\x18\x01 \x03(\x0c\"U\n\x15PullFileResultRequest\x12\r\n\x05jobId\x18\x01 \x01(\t\x12\x10\n\x08metadata\x18\x02 \x01(\t\x12\x0f\n\x05\x61sNpz\x18\x04 \x01(\x08H\x00\x42\n\n\x08\x66ileType\"\xaa\x01\n\x12\x46ileResultResponse\x12\r\n\x05jobId\x18\x01 \x01(\t\x12\x16\n\x0e\x63ontrollerName\x18\x02 \x01(\t\x12\r\n\x05group\x18\x03 \x01(\t\x12\r\n\x03npz\x18\x04 \x01(\x0cH\x00\x12\x0f\n\x07version\x18\n \x01(\t\x12\x36\n\x06\x65rrors\x18\x0b \x03(\x0b\x32&.qm.grpc.general_messages.ErrorMessageB\x06\n\x04\x66ile\"\x97\x01\n\x1bPullSimulatorSamplesRequest\x12\r\n\x05jobId\x18\x01 \x01(\t\x12\x15\n\rincludeAnalog\x18\x02 \x01(\x08\x12\x16\n\x0eincludeDigital\x18\x03 \x01(\x08\x12\x0f\n\x05\x61sNpz\x18\x04 \x01(\x08H\x00\x12\x1d\n\x15includeAllConnections\x18\x05 \x01(\x08\x42\n\n\x08\x66ileType\"\xa4\x02\n\x18SimulatorSamplesResponse\x12\r\n\x05jobId\x18\x01 \x01(\t\x12\n\n\x02ok\x18\x05 \x01(\x08\x12K\n\x06header\x18\x06 \x01(\x0b\x32\x39.qm.grpc.results_analyser.SimulatorSamplesResponse.HeaderH\x00\x12G\n\x04\x64\x61ta\x18\x07 \x01(\x0b\x32\x37.qm.grpc.results_analyser.SimulatorSamplesResponse.DataH\x00\x1a\x33\n\x06Header\x12\x13\n\x0bsimpleDType\x18\x01 \x01(\t\x12\x14\n\x0c\x63ountOfItems\x18\x02 \x01(\x05\x1a\x14\n\x04\x44\x61ta\x12\x0c\n\x04\x64\x61ta\x18\x08 \x01(\x0c\x42\x06\n\x04\x62odyJ\x04\x08\x02\x10\x05\"\x93\x01\n\rIcpResultData\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x11\n\ttimestamp\x18\x02 \x01(\x04\x12\x10\n\x08\x64\x61taLoss\x18\x03 \x01(\x08\x12\x12\n\x08intValue\x18\x0b \x01(\x05H\x00\x12\x15\n\x0b\x64oubleValue\x18\x0c \x01(\x01H\x00\x12\x16\n\x0c\x62ooleanValue\x18\r \x01(\x08H\x00\x42\x06\n\x04\x64\x61taJ\x04\x08\x04\x10\x0b\"\x8a\x01\n\x10StreamResultData\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x11\n\ttimestamp\x18\x02 \x01(\x04\x12\x10\n\x08\x64\x61taLoss\x18\x03 \x01(\x08\x12\x17\n\x0fmultipleSources\x18\x04 \x01(\x08\x12\x16\n\x0e\x64\x61taSourceName\x18\x05 \x01(\t\x12\x0c\n\x04\x64\x61ta\x18\x0b \x01(\x03J\x04\x08\x06\x10\x0b\x32\x8c\x05\n\x11JobResultsService\x12\x81\x01\n\x12GetJobResultSchema\x12\x33.qm.grpc.results_analyser.GetJobResultSchemaRequest\x1a\x34.qm.grpc.results_analyser.GetJobResultSchemaResponse\"\x00\x12l\n\x0bGetJobState\x12,.qm.grpc.results_analyser.GetJobStateRequest\x1a-.qm.grpc.results_analyser.GetJobStateResponse\"\x00\x12\x90\x01\n\x17GetJobNamedResultHeader\x12\x38.qm.grpc.results_analyser.GetJobNamedResultHeaderRequest\x1a\x39.qm.grpc.results_analyser.GetJobNamedResultHeaderResponse\"\x00\x12\x80\x01\n\x11GetJobNamedResult\x12\x32.qm.grpc.results_analyser.GetJobNamedResultRequest\x1a\x33.qm.grpc.results_analyser.GetJobNamedResultResponse\"\x00\x30\x01\x12o\n\x0cGetJobErrors\x12-.qm.grpc.results_analyser.GetJobErrorsRequest\x1a..qm.grpc.results_analyser.GetJobErrorsResponse\"\x00\x42*\n\x0fqm.grpc.resultsB\x0fJobResultsProtoP\x01\xa2\x02\x03HLWb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qm.pb.job_results_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\017qm.grpc.resultsB\017JobResultsProtoP\001\242\002\003HLW'
  _globals['_GETJOBNAMEDRESULTHEADERRESPONSE'].fields_by_name['done']._loaded_options = None
  _globals['_GETJOBNAMEDRESULTHEADERRESPONSE'].fields_by_name['done']._serialized_options = b'\030\001'
  _globals['_GETJOBNAMEDRESULTHEADERRESPONSE'].fields_by_name['closed']._loaded_options = None
  _globals['_GETJOBNAMEDRESULTHEADERRESPONSE'].fields_by_name['closed']._serialized_options = b'\030\001'
  _globals['_GETJOBNAMEDRESULTREQUEST'].fields_by_name['offset']._loaded_options = None
  _globals['_GETJOBNAMEDRESULTREQUEST'].fields_by_name['offset']._serialized_options = b'\030\001'
  _globals['_GETJOBRESULTSCHEMAREQUEST']._serialized_start=145
  _globals['_GETJOBRESULTSCHEMAREQUEST']._serialized_end=187
  _globals['_GETJOBRESULTSCHEMARESPONSE']._serialized_start=190
  _globals['_GETJOBRESULTSCHEMARESPONSE']._serialized_end=391
  _globals['_GETJOBRESULTSCHEMARESPONSE_ITEM']._serialized_start=294
  _globals['_GETJOBRESULTSCHEMARESPONSE_ITEM']._serialized_end=391
  _globals['_GETJOBSTATEREQUEST']._serialized_start=393
  _globals['_GETJOBSTATEREQUEST']._serialized_end=428
  _globals['_GETJOBSTATERESPONSE']._serialized_start=430
  _globals['_GETJOBSTATERESPONSE']._serialized_end=502
  _globals['_GETJOBNAMEDRESULTHEADERREQUEST']._serialized_start=504
  _globals['_GETJOBNAMEDRESULTHEADERREQUEST']._serialized_end=591
  _globals['_GETJOBNAMEDRESULTHEADERRESPONSE']._serialized_start=594
  _globals['_GETJOBNAMEDRESULTHEADERRESPONSE']._serialized_end=816
  _globals['_GETJOBNAMEDRESULTREQUEST']._serialized_start=819
  _globals['_GETJOBNAMEDRESULTREQUEST']._serialized_end=964
  _globals['_GETJOBNAMEDRESULTRESPONSE']._serialized_start=966
  _globals['_GETJOBNAMEDRESULTRESPONSE']._serialized_end=1029
  _globals['_GETJOBERRORSREQUEST']._serialized_start=1031
  _globals['_GETJOBERRORSREQUEST']._serialized_end=1067
  _globals['_GETJOBERRORSRESPONSE']._serialized_start=1070
  _globals['_GETJOBERRORSRESPONSE']._serialized_end=1367
  _globals['_GETJOBERRORSRESPONSE_ERROR']._serialized_start=1180
  _globals['_GETJOBERRORSRESPONSE_ERROR']._serialized_end=1317
  _globals['_GETJOBERRORSRESPONSE_EXECUTIONERRORSEVERITY']._serialized_start=1319
  _globals['_GETJOBERRORSRESPONSE_EXECUTIONERRORSEVERITY']._serialized_end=1367
  _globals['_PULLANALYSEDRESULTSREQUEST']._serialized_start=1369
  _globals['_PULLANALYSEDRESULTSREQUEST']._serialized_end=1461
  _globals['_ANALYSEDRESULTSRESPONSE']._serialized_start=1464
  _globals['_ANALYSEDRESULTSRESPONSE']._serialized_end=1696
  _globals['_PULLRESULTOUTPUTREQUEST']._serialized_start=1698
  _globals['_PULLRESULTOUTPUTREQUEST']._serialized_end=1758
  _globals['_PULLRESULTOUTPUTRESPONSE']._serialized_start=1761
  _globals['_PULLRESULTOUTPUTRESPONSE']._serialized_end=2347
  _globals['_PULLRESULTOUTPUTRESPONSE_HEADER']._serialized_start=2082
  _globals['_PULLRESULTOUTPUTRESPONSE_HEADER']._serialized_end=2217
  _globals['_PULLRESULTOUTPUTRESPONSE_DATA']._serialized_start=2219
  _globals['_PULLRESULTOUTPUTRESPONSE_DATA']._serialized_end=2266
  _globals['_PULLRESULTOUTPUTRESPONSE_NPZ']._serialized_start=2268
  _globals['_PULLRESULTOUTPUTRESPONSE_NPZ']._serialized_end=2286
  _globals['_PULLRESULTOUTPUTRESPONSE_NPY']._serialized_start=2288
  _globals['_PULLRESULTOUTPUTRESPONSE_NPY']._serialized_end=2335
  _globals['_BINARYWRAPPER']._serialized_start=2349
  _globals['_BINARYWRAPPER']._serialized_end=2378
  _globals['_PULLFILERESULTREQUEST']._serialized_start=2380
  _globals['_PULLFILERESULTREQUEST']._serialized_end=2465
  _globals['_FILERESULTRESPONSE']._serialized_start=2468
  _globals['_FILERESULTRESPONSE']._serialized_end=2638
  _globals['_PULLSIMULATORSAMPLESREQUEST']._serialized_start=2641
  _globals['_PULLSIMULATORSAMPLESREQUEST']._serialized_end=2792
  _globals['_SIMULATORSAMPLESRESPONSE']._serialized_start=2795
  _globals['_SIMULATORSAMPLESRESPONSE']._serialized_end=3087
  _globals['_SIMULATORSAMPLESRESPONSE_HEADER']._serialized_start=3000
  _globals['_SIMULATORSAMPLESRESPONSE_HEADER']._serialized_end=3051
  _globals['_SIMULATORSAMPLESRESPONSE_DATA']._serialized_start=3053
  _globals['_SIMULATORSAMPLESRESPONSE_DATA']._serialized_end=3073
  _globals['_ICPRESULTDATA']._serialized_start=3090
  _globals['_ICPRESULTDATA']._serialized_end=3237
  _globals['_STREAMRESULTDATA']._serialized_start=3240
  _globals['_STREAMRESULTDATA']._serialized_end=3378
  _globals['_JOBRESULTSSERVICE']._serialized_start=3381
  _globals['_JOBRESULTSSERVICE']._serialized_end=4033
# @@protoc_insertion_point(module_scope)



from typing import Union, cast, overload

from qm.qua._dsl.variable_handling import save
from qm.exceptions import QmQuaException, SaveToAdcTraceException
from qm.qua._dsl.streams.external_streams import _send_to_opnic_stream
from qm.qua._dsl.stream_processing.stream_processing import ResultStreamSource
from qm.qua._expressions import (
    StructT,
    ScalarOfAnyType,
    OutputStreamInterface,
    QuaExternalOutgoingStream,
    validate_scalar_of_any_type,
)


@overload
def send_to_stream(stream: ResultStreamSource, data: ScalarOfAnyType) -> None:
    """
    Send a scalar value to a client output stream.

    This is equivalent to [qm.qua.save][] with a declared client output stream:
    ``send_to_stream(stream, value)`` and ``save(value, stream)`` produce the same stream items.

    Args:
        stream (ResultStreamSource): The client output stream to write to.
        data (ScalarOfAnyType): The scalar value to send.
    """
    ...


@overload
def send_to_stream(stream: QuaExternalOutgoingStream[StructT], data: StructT) -> None:
    """
    Send one packet to an outgoing OPNIC stream.

    Args:
        stream (QuaExternalOutgoingStream[StructT]): The outgoing OPNIC stream to write to.
        data (StructT): The struct instance to send. It must match the packet type declared for the stream.
    """
    ...


def send_to_stream(stream: OutputStreamInterface, data: Union[StructT, ScalarOfAnyType]) -> None:
    """
    Send one item to an output stream.

    For client output streams declared with [qm.qua.declare_output_stream][], this is equivalent to
    [qm.qua.save][] with a declared stream:
    ``send_to_stream(stream, value)`` and ``save(value, stream)`` produce the same client-stream items.
    If the stream was declared with ``dtype``, the value is cast to that type before it is saved.

    Writing to a declared client output stream does not by itself create a client-visible result handle.
    The stream items become input to [qm.qua.stream_processing][], and to retrieve them on the client
    you must terminate the pipeline in a ``with stream_processing():`` block, for example with
    ``my_stream.save_all("results")``.

    This function does not replace the legacy tag-based form ``save(value, "tag")``, which still writes
    directly to a result tag without requiring a declared client output stream.

    For OPNIC output streams, this sends one packet represented by a struct instance to the OPNIC endpoint.

    Args:
        stream: The outgoing stream to send data to.
        data: The data to send. Client streams accept only scalar values. OPNIC streams require a struct
            instance that matches the packet type declared for the stream.

    Note:
        ``send_to_stream()`` cannot be used with ADC-trace streams. Streams used with
        ``measure(..., adc_stream=...)`` should not be used with ``send_to_stream()``.

    Example:
        ```python
        # Client usage example
        result_stream = declare_output_stream("client", dtype=int)
        send_to_stream(result_stream, 3.14)

        # OPNIC usage example
        packet_stream = declare_output_stream("opnic", 2, Packet)
        packet = declare_struct(Packet)
        send_to_stream(packet_stream, packet)
        ```
    """
    if isinstance(stream, QuaExternalOutgoingStream):
        # Validation of data type is done inside _send_to_opnic_stream
        _send_to_opnic_stream(stream, data)

    elif isinstance(stream, ResultStreamSource):
        validate_scalar_of_any_type(data)
        try:
            if stream.dtype is not None:
                data = stream.dtype(data)  # Casting to the data type that was configured for the stream
            save(cast(ScalarOfAnyType, data), stream)
        except SaveToAdcTraceException:
            raise SaveToAdcTraceException("`send` cannot be used to for adc_trace streams.")
    else:
        raise QmQuaException(f"Unsupported stream type: {type(stream).__name__}.")



import logging
import contextlib
from abc import ABCMeta, abstractmethod
from typing import (
    Any,
    Type,
    Tuple,
    Union,
    Generic,
    TypeVar,
    Callable,
    Iterator,
    Optional,
    Protocol,
    Sequence,
    Generator,
    runtime_checkable,
)

import grpc
from grpc import Channel
from google.protobuf.empty_pb2 import Empty
from google.protobuf.message import Message

from qm.utils.general_utils import is_debug
from qm.api.models.server_details import ConnectionDetails
from qm.exceptions import QMTimeoutError, QMConnectionError

logger = logging.getLogger(__name__)

Metadata = Sequence[Tuple[str, str]]


class GatewayNotImplementedError(NotImplementedError):
    pass


def timeout_error_message(timeout: Optional[float]) -> str:
    return f"A timeout of {timeout} seconds was reached. The timeout value can be configured either in the relevant API call (if supported) or when creating the QuantumMachinesManager instance."


@contextlib.contextmanager
def _handle_connection_error(timeout: Optional[float]) -> Generator[None, None, None]:
    try:
        yield
    except grpc.RpcError as e:
        if is_debug():
            logger.exception("Encountered connection error from QOP")

        # Get status code from gRPC exception
        status_code = e.code() if hasattr(e, "code") else None
        details = e.details() if hasattr(e, "details") else str(e)

        if status_code == grpc.StatusCode.UNIMPLEMENTED:
            raise GatewayNotImplementedError(
                f"Encountered connection error from QOP:  details: {details}, status: {status_code}"
            ) from e

        # Handle timeout specifically
        if status_code == grpc.StatusCode.DEADLINE_EXCEEDED:
            error_message = timeout_error_message(timeout)
            if is_debug():
                logger.exception(error_message)
            raise QMTimeoutError(error_message) from e

        raise QMConnectionError(
            f"Encountered connection error from QOP: details: {details}, status:  {status_code}"
        ) from e

    except TimeoutError as e:
        if is_debug():
            logger.exception(timeout_error_message(timeout))
        raise QMTimeoutError(timeout_error_message(timeout)) from e


@runtime_checkable
class GrpcStub(Protocol):
    def __init__(self, channel: Channel) -> None:
        ...

    ...


T = TypeVar("T")
StubType = TypeVar("StubType", bound=GrpcStub)
RequestMessageType = TypeVar("RequestMessageType", bound=Message)
ResponseMessageType = TypeVar("ResponseMessageType", bound=Message)
MetadataLike = Sequence[Tuple[str, str]]


class BaseApi(Generic[StubType], metaclass=ABCMeta):
    def __init__(self, connection_details: ConnectionDetails):
        self._connection_details = connection_details

        self._channel = self._connection_details.channel

        self._stub: StubType = self._stub_class(self._channel)  # type: ignore[assignment]

        self._timeout: Optional[float] = self._connection_details.timeout

    def _run(
        self,
        grpc_method: Callable[..., ResponseMessageType],
        request: Union[RequestMessageType, Empty],
        timeout: Optional[float] = None,
    ) -> ResponseMessageType:
        """
        Run a gRPC call and handle connection errors.

        Args:
            grpc_method: A method from the `self._stub` class (e.g., self._stub.SomeMethod)
            request: The request message to send
            timeout:  A duplicate of the timeout parameter provided to the `self._stub` function. If the default
            `self._timeout` is used, this parameter can be left empty.

        Returns:
            The response message from the gRPC call
        """
        if timeout is None:
            timeout = self._timeout

        with _handle_connection_error(timeout):
            return grpc_method(request, timeout=timeout)

    def _run_iterator(
        self,
        stub_func: Callable[[Any], Iterator[ResponseMessageType]],
        # The parameters are typed as 'Any' because MyPy encounters issues when attempting to declare more specific types.
        request: RequestMessageType,
        *,
        timeout: Optional[float] = None,
        metadata: Optional[MetadataLike] = None,
    ) -> Iterator[ResponseMessageType]:
        """
        Run a function that returns an Iterator (primarily from the self._stub functions), and handle connection errors.

        Args:
            stub_func: A function from the `self._stub` class that returns an Iterator
            request:  The request message to be sent to the function
            timeout:  Timeout for the request in seconds
            metadata: Additional metadata to send with the request

        Returns:
            The Iterator of response messages from the function
        """

        with _handle_connection_error(timeout):
            for response in stub_func(request, timeout=timeout, metadata=metadata):  # type: ignore[call-arg]
                yield response

    @property
    def connection_details(self) -> ConnectionDetails:
        return self._connection_details

    @property
    @abstractmethod
    def _stub_class(self) -> Type[GrpcStub]:
        pass

    @property
    def channel(self) -> Channel:
        return self._channel



import warnings
from enum import Enum
from typing import Union, Literal, overload

from qm._loc import _get_loc
from qm.grpc.qm.pb import inc_qua_pb2
from qm.exceptions import QmQuaException
from qm.utils import deprecation_message
from qm.api.models.capabilities import QopCaps
from qm.qua._scope_management.scopes_manager import scopes_manager
from qm.qua._expressions import StructT, QuaExternalIncomingStream, QuaExternalOutgoingStream


class QuaStreamDirection(Enum):
    INCOMING = "INCOMING"
    OUTGOING = "OUTGOING"


@overload
def declare_external_stream(
    struct_type: type[StructT], stream_id: int, direction: Literal[QuaStreamDirection.INCOMING]
) -> QuaExternalIncomingStream[StructT]:
    ...


@overload
def declare_external_stream(
    struct_type: type[StructT], stream_id: int, direction: Literal[QuaStreamDirection.OUTGOING]
) -> QuaExternalOutgoingStream[StructT]:
    ...


def declare_external_stream(
    struct_type: type[StructT], stream_id: int, direction: QuaStreamDirection
) -> Union[QuaExternalIncomingStream[StructT], QuaExternalOutgoingStream[StructT]]:
    """Declare a stream to an external compute resource.

    The stream can be either an incoming or an outgoing stream, which will receive or send data from the external
    compute resource respectively.

    Args:
        struct_type: A QuaStruct type that defines a single packet of the stream.
        stream_id: The ID of the stream, an integer between 0 and 1000.
            Has to match the ID of the stream declared in
            the external compute resource.
        direction: The direction of the stream, either QuaStreamDirection.INCOMING or QuaStreamDirection.OUTGOING.

    Example:
        ```python
        struct = declare_struct(StructType)
        stream = declare_external_stream(StructType, 0, QuaStreamDirection.INCOMING)
        ```

    """
    warnings.warn(
        deprecation_message(
            "declare_external_stream",
            deprecated_in="1.2.4",
            removed_in="2.0.0",
            details="Please use declare_input_stream() / declare_output_stream() instead.",
        ),
        DeprecationWarning,
        stacklevel=2,
    )
    return _declare_opnic_stream(struct_type, stream_id, direction)


def _declare_opnic_stream(
    struct_type: type[StructT], stream_id: int, direction: QuaStreamDirection
) -> Union[QuaExternalIncomingStream[StructT], QuaExternalOutgoingStream[StructT]]:
    program_scope = scopes_manager.program_scope
    program_scope.add_used_capability(QopCaps.external_stream)

    if not isinstance(struct_type, type):
        raise QmQuaException(f"struct_type must be a type (class), got {type(struct_type).__name__}")

    if not hasattr(struct_type, "__members_initializers__"):
        raise QmQuaException(
            f"struct_type must be a QuaStruct type (decorated with @qua_struct), got {struct_type.__name__ if hasattr(struct_type, '__name__') else type(struct_type).__name__}"
        )

    if not 1 <= stream_id <= 1023:
        raise QmQuaException("stream_id must be between 1 and 1023")

    stream: Union[QuaExternalIncomingStream[StructT], QuaExternalOutgoingStream[StructT]]

    stream_identifier = (stream_id, direction)
    if stream_identifier in program_scope.declared_external_streams:
        raise QmQuaException(f"Opnic stream with id '{stream_id}' already declared")

    if direction == QuaStreamDirection.INCOMING:
        stream = QuaExternalIncomingStream(stream_id, struct_type)
    elif direction == QuaStreamDirection.OUTGOING:
        stream = QuaExternalOutgoingStream(stream_id, struct_type)
    else:
        raise QmQuaException("direction must be either QuaStreamDirection.INCOMING or QuaStreamDirection.OUTGOING")

    program_scope.add_external_stream_declaration(stream_identifier, stream.declaration_statement)
    return stream


def send_to_external_stream(stream: QuaExternalOutgoingStream[StructT], struct: StructT) -> None:
    """
    Send data to an external compute resource.

    Args:
        stream: The outgoing stream to send data to.
        struct: The struct containing the data to send.
    """
    warnings.warn(
        deprecation_message(
            "send_to_external_stream",
            deprecated_in="1.2.4",
            removed_in="2.0.0",
            details="Please use send_to_stream() instead.",
        ),
        DeprecationWarning,
        stacklevel=2,
    )
    _send_to_opnic_stream(stream, struct)


def _send_to_opnic_stream(stream: QuaExternalOutgoingStream[StructT], struct: StructT) -> None:
    try:
        _ = struct.struct_reference
    except AttributeError:
        raise QmQuaException("Data sent to an opnic stream must be a struct instance.")

    statement = inc_qua_pb2.QuaProgram.AnyStatement(
        sendToExternalStream=inc_qua_pb2.QuaProgram.SendToExternalStreamStatement(
            loc=_get_loc(), stream=stream.unwrapped, struct=struct.struct_reference.unwrapped
        )
    )
    scopes_manager.append_statement(statement)


def receive_from_external_stream(stream: QuaExternalIncomingStream[StructT], struct: StructT) -> None:
    """
    Receive data from an external compute resource.

    Args:
        stream: The incoming stream to receive data from.
        struct: The struct to store the received data in.
    """
    warnings.warn(
        deprecation_message(
            "receive_from_external_stream",
            deprecated_in="1.2.4",
            removed_in="2.0.0",
            details="Please use receive_from_stream() instead.",
        ),
        DeprecationWarning,
        stacklevel=2,
    )
    _receive_from_opnic_stream(stream, struct)


def _receive_from_opnic_stream(stream: QuaExternalIncomingStream[StructT], struct: StructT) -> None:
    try:
        _ = struct.struct_reference
    except AttributeError:
        raise QmQuaException(
            f"Data received from an opnic stream must be saved into a struct instance. got {type(struct).__name__} instead."
        )

    statement = inc_qua_pb2.QuaProgram.AnyStatement(
        receiveFromExternalStream=inc_qua_pb2.QuaProgram.ReceiveFromExternalStreamStatement(
            loc=_get_loc(), stream=stream.unwrapped, struct=struct.struct_reference.unwrapped
        )
    )
    scopes_manager.append_statement(statement)



from qm.grpc.qm.pb import inc_qua_config_pb2 as _inc_qua_config_pb2
from qm.grpc.qm.pb import general_messages_pb2 as _general_messages_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class OpenQuantumMachineRequest(_message.Message):
    __slots__ = ("config", "never", "always", "ifNeeded", "keep_dc_offsets_when_closing")
    CONFIG_FIELD_NUMBER: _ClassVar[int]
    NEVER_FIELD_NUMBER: _ClassVar[int]
    ALWAYS_FIELD_NUMBER: _ClassVar[int]
    IFNEEDED_FIELD_NUMBER: _ClassVar[int]
    KEEP_DC_OFFSETS_WHEN_CLOSING_FIELD_NUMBER: _ClassVar[int]
    config: _inc_qua_config_pb2.QuaConfig
    never: bool
    always: bool
    ifNeeded: bool
    keep_dc_offsets_when_closing: bool
    def __init__(self, config: _Optional[_Union[_inc_qua_config_pb2.QuaConfig, _Mapping]] = ..., never: bool = ..., always: bool = ..., ifNeeded: bool = ..., keep_dc_offsets_when_closing: bool = ...) -> None: ...

class OpenQuantumMachineResponse(_message.Message):
    __slots__ = ("machineID", "success", "configValidationErrors", "physicalValidationErrors", "openQmWarnings")
    MACHINEID_FIELD_NUMBER: _ClassVar[int]
    SUCCESS_FIELD_NUMBER: _ClassVar[int]
    CONFIGVALIDATIONERRORS_FIELD_NUMBER: _ClassVar[int]
    PHYSICALVALIDATIONERRORS_FIELD_NUMBER: _ClassVar[int]
    OPENQMWARNINGS_FIELD_NUMBER: _ClassVar[int]
    machineID: str
    success: bool
    configValidationErrors: _containers.RepeatedCompositeFieldContainer[ConfigValidationMessage]
    physicalValidationErrors: _containers.RepeatedCompositeFieldContainer[PhysicalValidationMessage]
    openQmWarnings: _containers.RepeatedCompositeFieldContainer[OpenQmWarning]
    def __init__(self, machineID: _Optional[str] = ..., success: bool = ..., configValidationErrors: _Optional[_Iterable[_Union[ConfigValidationMessage, _Mapping]]] = ..., physicalValidationErrors: _Optional[_Iterable[_Union[PhysicalValidationMessage, _Mapping]]] = ..., openQmWarnings: _Optional[_Iterable[_Union[OpenQmWarning, _Mapping]]] = ...) -> None: ...

class CloseQuantumMachineRequest(_message.Message):
    __slots__ = ("machineID",)
    MACHINEID_FIELD_NUMBER: _ClassVar[int]
    machineID: str
    def __init__(self, machineID: _Optional[str] = ...) -> None: ...

class CloseQuantumMachineResponse(_message.Message):
    __slots__ = ("success", "errors")
    SUCCESS_FIELD_NUMBER: _ClassVar[int]
    ERRORS_FIELD_NUMBER: _ClassVar[int]
    success: bool
    errors: _containers.RepeatedCompositeFieldContainer[_general_messages_pb2.ErrorMessage]
    def __init__(self, success: bool = ..., errors: _Optional[_Iterable[_Union[_general_messages_pb2.ErrorMessage, _Mapping]]] = ...) -> None: ...

class GetQuantumMachineRequest(_message.Message):
    __slots__ = ("machineID",)
    MACHINEID_FIELD_NUMBER: _ClassVar[int]
    machineID: str
    def __init__(self, machineID: _Optional[str] = ...) -> None: ...

class GetQuantumMachineResponse(_message.Message):
    __slots__ = ("machineID", "config", "success", "errors")
    MACHINEID_FIELD_NUMBER: _ClassVar[int]
    CONFIG_FIELD_NUMBER: _ClassVar[int]
    SUCCESS_FIELD_NUMBER: _ClassVar[int]
    ERRORS_FIELD_NUMBER: _ClassVar[int]
    machineID: str
    config: _inc_qua_config_pb2.QuaConfig
    success: bool
    errors: _containers.RepeatedCompositeFieldContainer[_general_messages_pb2.ErrorMessage]
    def __init__(self, machineID: _Optional[str] = ..., config: _Optional[_Union[_inc_qua_config_pb2.QuaConfig, _Mapping]] = ..., success: bool = ..., errors: _Optional[_Iterable[_Union[_general_messages_pb2.ErrorMessage, _Mapping]]] = ...) -> None: ...

class GetRunningJobRequest(_message.Message):
    __slots__ = ("machineID",)
    MACHINEID_FIELD_NUMBER: _ClassVar[int]
    machineID: str
    def __init__(self, machineID: _Optional[str] = ...) -> None: ...

class GetRunningJobResponse(_message.Message):
    __slots__ = ("machineID", "jobId")
    MACHINEID_FIELD_NUMBER: _ClassVar[int]
    JOBID_FIELD_NUMBER: _ClassVar[int]
    machineID: str
    jobId: str
    def __init__(self, machineID: _Optional[str] = ..., jobId: _Optional[str] = ...) -> None: ...

class ListOpenQuantumMachinesResponse(_message.Message):
    __slots__ = ("machineIDs",)
    MACHINEIDS_FIELD_NUMBER: _ClassVar[int]
    machineIDs: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, machineIDs: _Optional[_Iterable[str]] = ...) -> None: ...

class CloseAllQuantumMachinesResponse(_message.Message):
    __slots__ = ("success", "errors")
    SUCCESS_FIELD_NUMBER: _ClassVar[int]
    ERRORS_FIELD_NUMBER: _ClassVar[int]
    success: bool
    errors: _containers.RepeatedCompositeFieldContainer[_general_messages_pb2.ErrorMessage]
    def __init__(self, success: bool = ..., errors: _Optional[_Iterable[_Union[_general_messages_pb2.ErrorMessage, _Mapping]]] = ...) -> None: ...

class GetControllersResponse(_message.Message):
    __slots__ = ("controllers",)
    CONTROLLERS_FIELD_NUMBER: _ClassVar[int]
    controllers: _containers.RepeatedCompositeFieldContainer[Controller]
    def __init__(self, controllers: _Optional[_Iterable[_Union[Controller, _Mapping]]] = ...) -> None: ...

class Controller(_message.Message):
    __slots__ = ("name", "temperature")
    NAME_FIELD_NUMBER: _ClassVar[int]
    TEMPERATURE_FIELD_NUMBER: _ClassVar[int]
    name: str
    temperature: float
    def __init__(self, name: _Optional[str] = ..., temperature: _Optional[float] = ...) -> None: ...

class ConfigValidationMessage(_message.Message):
    __slots__ = ("message", "group", "path", "level")
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    GROUP_FIELD_NUMBER: _ClassVar[int]
    PATH_FIELD_NUMBER: _ClassVar[int]
    LEVEL_FIELD_NUMBER: _ClassVar[int]
    message: str
    group: str
    path: str
    level: _general_messages_pb2.MessageLevel
    def __init__(self, message: _Optional[str] = ..., group: _Optional[str] = ..., path: _Optional[str] = ..., level: _Optional[_Union[_general_messages_pb2.MessageLevel, str]] = ...) -> None: ...

class PhysicalValidationMessage(_message.Message):
    __slots__ = ("message", "group", "path", "level")
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    GROUP_FIELD_NUMBER: _ClassVar[int]
    PATH_FIELD_NUMBER: _ClassVar[int]
    LEVEL_FIELD_NUMBER: _ClassVar[int]
    message: str
    group: str
    path: str
    level: _general_messages_pb2.MessageLevel
    def __init__(self, message: _Optional[str] = ..., group: _Optional[str] = ..., path: _Optional[str] = ..., level: _Optional[_Union[_general_messages_pb2.MessageLevel, str]] = ...) -> None: ...

class OpenQmWarning(_message.Message):
    __slots__ = ("code", "message")
    CODE_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    code: int
    message: str
    def __init__(self, code: _Optional[int] = ..., message: _Optional[str] = ...) -> None: ...



from copy import copy
from abc import ABC, abstractmethod
from typing import Any, Dict, List, Type, Generic, Mapping, TypeVar, Optional, Collection, cast

from google.protobuf.message import Message

from qm.exceptions import ConfigValidationException
from qm.utils.protobuf_utils import assign_to_proto
from qm.api.models.capabilities import QopCaps, ServerCapabilities

InputType = TypeVar("InputType", bound=Mapping[str, Any])
OutputType = TypeVar("OutputType", bound=Message)
T = TypeVar("T", bound=Mapping[str, Any])


class BaseDictToPbConverter(Generic[InputType, OutputType], ABC):
    def __init__(self, capabilities: ServerCapabilities, init_mode: bool) -> None:
        self._capabilities = capabilities
        self._init_mode = init_mode

    @abstractmethod
    def convert(self, input_data: InputType) -> OutputType:
        pass

    @abstractmethod
    def deconvert(self, output_data: OutputType) -> InputType:
        pass

    def _apply_defaults(
        self,
        config: T,
        default_schema: T,
    ) -> T:
        """
        Merge default values into the configuration dictionary, if applicable.

        If `init_mode` is True, missing keys from `config` will be filled in using `default_schema`.
        If `init_mode` is False and the server supports `config_v2`, the original `config` is returned unchanged (in
        config_v1 defaults will always be applied).

        Args:
            config (T): A user-defined config, possibly missing some keys.
            default_schema (T): Schema with default values for keys (possibly missing some keys).

        Returns:
            A new config dictionary with defaults applied, or the original config.
        """

        if not self._init_mode and self._capabilities.supports(QopCaps.config_v2):
            return config

        # The casting is that mypy will allow the update method
        new_config = cast(Dict[Any, Any], copy(default_schema))
        new_config.update(config)
        return cast(T, new_config)

    def _set_pb_attr_config_v2(
        self,
        item: Message,
        value: Any,
        v1_attr: str,
        v2_attr: str,
        allow_nones: bool = False,
        create_container: Optional[Type[Message]] = None,
    ) -> None:
        if not hasattr(item, v1_attr) or not hasattr(item, v2_attr):
            raise AttributeError(f"Either {v1_attr} or {v2_attr} do not exist in {item}")

        if value is None and not allow_nones:
            return

        if self._capabilities.supports(QopCaps.config_v2):
            container_message = getattr(item, v2_attr)

            if container_message is None and create_container:
                container_message = create_container()
                setattr(item, v2_attr, container_message)

            if value is not None:
                if not hasattr(container_message, "value"):
                    raise AttributeError(f"{v2_attr} does not have a 'value' attribute")
                assign_to_proto(container_message, "value", value)
            else:
                item.ClearField(v2_attr)
        else:
            if value:
                assign_to_proto(item, v1_attr, value)
            else:
                item.ClearField(v1_attr)

    @staticmethod
    def _validate_required_fields(config: Mapping[str, Any], fields: List[str], parent_field: str) -> None:
        for field in fields:
            if field not in config:
                raise ConfigValidationException(f"{field} should be declared when initializing a {parent_field}")

    @staticmethod
    def _validate_unsupported_params(
        data: Collection[str],
        unsupported_params: Collection[str],
        supported_params: Collection[str] = (),
        supported_from: Optional[str] = None,
        supported_until: Optional[str] = None,
    ) -> None:
        if set(data) & set(unsupported_params):
            if supported_from:
                unsupported_message = f"supported only from QOP {supported_from} and later"
            elif supported_until:
                unsupported_message = f"supported only until QOP {supported_until}"
            else:
                raise ValueError("Either 'supported_from' or 'supported_until' must be provided.")

            error_message = f"The configuration keys {unsupported_params} are {unsupported_message}."
            if supported_params:
                error_message += f" Use the keys {supported_params} instead."

            raise ConfigValidationException(error_message)



from qm.grpc.qm.pb import inc_qua_pb2 as _inc_qua_pb2
from qm.grpc.qm.pb import inc_qua_config_pb2 as _inc_qua_config_pb2
from qm.grpc.qm.pb import inc_qm_api_pb2 as _inc_qm_api_pb2
from google.protobuf import struct_pb2 as _struct_pb2
from google.protobuf import empty_pb2 as _empty_pb2
from google.protobuf import wrappers_pb2 as _wrappers_pb2
from qm.grpc.qm.pb import compiler_pb2 as _compiler_pb2
from qm.grpc.qm.pb import job_results_pb2 as _job_results_pb2
from qm.grpc.qm.pb import qm_manager_pb2 as _qm_manager_pb2
from qm.grpc.qm.pb import general_messages_pb2 as _general_messages_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class QueuePosition(_message.Message):
    __slots__ = ("end", "start")
    END_FIELD_NUMBER: _ClassVar[int]
    START_FIELD_NUMBER: _ClassVar[int]
    end: _empty_pb2.Empty
    start: _empty_pb2.Empty
    def __init__(self, end: _Optional[_Union[_empty_pb2.Empty, _Mapping]] = ..., start: _Optional[_Union[_empty_pb2.Empty, _Mapping]] = ...) -> None: ...

class AddToQueueRequest(_message.Message):
    __slots__ = ("quantumMachineId", "highLevelProgram", "queuePosition")
    QUANTUMMACHINEID_FIELD_NUMBER: _ClassVar[int]
    HIGHLEVELPROGRAM_FIELD_NUMBER: _ClassVar[int]
    QUEUEPOSITION_FIELD_NUMBER: _ClassVar[int]
    quantumMachineId: str
    highLevelProgram: _inc_qua_pb2.QuaProgram
    queuePosition: QueuePosition
    def __init__(self, quantumMachineId: _Optional[str] = ..., highLevelProgram: _Optional[_Union[_inc_qua_pb2.QuaProgram, _Mapping]] = ..., queuePosition: _Optional[_Union[QueuePosition, _Mapping]] = ...) -> None: ...

class AddToQueueResponse(_message.Message):
    __slots__ = ("ok", "jobId", "messages")
    OK_FIELD_NUMBER: _ClassVar[int]
    JOBID_FIELD_NUMBER: _ClassVar[int]
    MESSAGES_FIELD_NUMBER: _ClassVar[int]
    ok: bool
    jobId: str
    messages: _containers.RepeatedCompositeFieldContainer[_compiler_pb2.CompilerMessage]
    def __init__(self, ok: bool = ..., jobId: _Optional[str] = ..., messages: _Optional[_Iterable[_Union[_compiler_pb2.CompilerMessage, _Mapping]]] = ...) -> None: ...

class WaveformOverride(_message.Message):
    __slots__ = ("samples",)
    SAMPLES_FIELD_NUMBER: _ClassVar[int]
    samples: _containers.RepeatedScalarFieldContainer[float]
    def __init__(self, samples: _Optional[_Iterable[float]] = ...) -> None: ...

class ExecutionOverrides(_message.Message):
    __slots__ = ("waveforms",)
    class WaveformsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: WaveformOverride
        def __init__(self, key: _Optional[str] = ..., value: _Optional[_Union[WaveformOverride, _Mapping]] = ...) -> None: ...
    WAVEFORMS_FIELD_NUMBER: _ClassVar[int]
    waveforms: _containers.MessageMap[str, WaveformOverride]
    def __init__(self, waveforms: _Optional[_Mapping[str, WaveformOverride]] = ...) -> None: ...

class AddCompiledToQueueRequest(_message.Message):
    __slots__ = ("quantumMachineId", "programId", "queuePosition", "executionOverrides")
    QUANTUMMACHINEID_FIELD_NUMBER: _ClassVar[int]
    PROGRAMID_FIELD_NUMBER: _ClassVar[int]
    QUEUEPOSITION_FIELD_NUMBER: _ClassVar[int]
    EXECUTIONOVERRIDES_FIELD_NUMBER: _ClassVar[int]
    quantumMachineId: str
    programId: str
    queuePosition: QueuePosition
    executionOverrides: ExecutionOverrides
    def __init__(self, quantumMachineId: _Optional[str] = ..., programId: _Optional[str] = ..., queuePosition: _Optional[_Union[QueuePosition, _Mapping]] = ..., executionOverrides: _Optional[_Union[ExecutionOverrides, _Mapping]] = ...) -> None: ...

class AddCompiledToQueueResponse(_message.Message):
    __slots__ = ("ok", "jobId", "errors")
    OK_FIELD_NUMBER: _ClassVar[int]
    JOBID_FIELD_NUMBER: _ClassVar[int]
    ERRORS_FIELD_NUMBER: _ClassVar[int]
    ok: bool
    jobId: str
    errors: _containers.RepeatedCompositeFieldContainer[_general_messages_pb2.ErrorMessage]
    def __init__(self, ok: bool = ..., jobId: _Optional[str] = ..., errors: _Optional[_Iterable[_Union[_general_messages_pb2.ErrorMessage, _Mapping]]] = ...) -> None: ...

class CompileRequest(_message.Message):
    __slots__ = ("quantumMachineId", "highLevelProgram")
    QUANTUMMACHINEID_FIELD_NUMBER: _ClassVar[int]
    HIGHLEVELPROGRAM_FIELD_NUMBER: _ClassVar[int]
    quantumMachineId: str
    highLevelProgram: _inc_qua_pb2.QuaProgram
    def __init__(self, quantumMachineId: _Optional[str] = ..., highLevelProgram: _Optional[_Union[_inc_qua_pb2.QuaProgram, _Mapping]] = ...) -> None: ...

class CompileResponse(_message.Message):
    __slots__ = ("ok", "programId", "messages")
    OK_FIELD_NUMBER: _ClassVar[int]
    PROGRAMID_FIELD_NUMBER: _ClassVar[int]
    MESSAGES_FIELD_NUMBER: _ClassVar[int]
    ok: bool
    programId: str
    messages: _containers.RepeatedCompositeFieldContainer[_compiler_pb2.CompilerMessage]
    def __init__(self, ok: bool = ..., programId: _Optional[str] = ..., messages: _Optional[_Iterable[_Union[_compiler_pb2.CompilerMessage, _Mapping]]] = ...) -> None: ...

class GetJobExecutionStatusRequest(_message.Message):
    __slots__ = ("quantumMachineId", "jobId")
    QUANTUMMACHINEID_FIELD_NUMBER: _ClassVar[int]
    JOBID_FIELD_NUMBER: _ClassVar[int]
    quantumMachineId: str
    jobId: str
    def __init__(self, quantumMachineId: _Optional[str] = ..., jobId: _Optional[str] = ...) -> None: ...

class GetJobExecutionStatusResponse(_message.Message):
    __slots__ = ("status",)
    STATUS_FIELD_NUMBER: _ClassVar[int]
    status: JobExecutionStatus
    def __init__(self, status: _Optional[_Union[JobExecutionStatus, _Mapping]] = ...) -> None: ...

class JobExecutionStatus(_message.Message):
    __slots__ = ("unknown", "pending", "running", "completed", "canceled", "loading", "error", "processing")
    class Unknown(_message.Message):
        __slots__ = ()
        def __init__(self) -> None: ...
    class Pending(_message.Message):
        __slots__ = ("positionInQueue", "timeAdded", "addedBy")
        POSITIONINQUEUE_FIELD_NUMBER: _ClassVar[int]
        TIMEADDED_FIELD_NUMBER: _ClassVar[int]
        ADDEDBY_FIELD_NUMBER: _ClassVar[int]
        positionInQueue: int
        timeAdded: _timestamp_pb2.Timestamp
        addedBy: str
        def __init__(self, positionInQueue: _Optional[int] = ..., timeAdded: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., addedBy: _Optional[str] = ...) -> None: ...
    class Running(_message.Message):
        __slots__ = ("timeAdded", "addedBy", "timeStarted")
        TIMEADDED_FIELD_NUMBER: _ClassVar[int]
        ADDEDBY_FIELD_NUMBER: _ClassVar[int]
        TIMESTARTED_FIELD_NUMBER: _ClassVar[int]
        timeAdded: _timestamp_pb2.Timestamp
        addedBy: str
        timeStarted: _timestamp_pb2.Timestamp
        def __init__(self, timeAdded: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., addedBy: _Optional[str] = ..., timeStarted: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...) -> None: ...
    class Completed(_message.Message):
        __slots__ = ("timeAdded", "addedBy", "timeStarted", "timeCompleted")
        TIMEADDED_FIELD_NUMBER: _ClassVar[int]
        ADDEDBY_FIELD_NUMBER: _ClassVar[int]
        TIMESTARTED_FIELD_NUMBER: _ClassVar[int]
        TIMECOMPLETED_FIELD_NUMBER: _ClassVar[int]
        timeAdded: _timestamp_pb2.Timestamp
        addedBy: str
        timeStarted: _timestamp_pb2.Timestamp
        timeCompleted: _timestamp_pb2.Timestamp
        def __init__(self, timeAdded: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., addedBy: _Optional[str] = ..., timeStarted: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., timeCompleted: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...) -> None: ...
    class Canceled(_message.Message):
        __slots__ = ("timeAdded", "addedBy", "timeCanceled")
        TIMEADDED_FIELD_NUMBER: _ClassVar[int]
        ADDEDBY_FIELD_NUMBER: _ClassVar[int]
        TIMECANCELED_FIELD_NUMBER: _ClassVar[int]
        timeAdded: _timestamp_pb2.Timestamp
        addedBy: str
        timeCanceled: _timestamp_pb2.Timestamp
        def __init__(self, timeAdded: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., addedBy: _Optional[str] = ..., timeCanceled: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...) -> None: ...
    class Loading(_message.Message):
        __slots__ = ("timeAdded", "addedBy")
        TIMEADDED_FIELD_NUMBER: _ClassVar[int]
        ADDEDBY_FIELD_NUMBER: _ClassVar[int]
        timeAdded: _timestamp_pb2.Timestamp
        addedBy: str
        def __init__(self, timeAdded: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., addedBy: _Optional[str] = ...) -> None: ...
    class Error(_message.Message):
        __slots__ = ("errorMessages", "addedBy")
        ERRORMESSAGES_FIELD_NUMBER: _ClassVar[int]
        ADDEDBY_FIELD_NUMBER: _ClassVar[int]
        errorMessages: _struct_pb2.ListValue
        addedBy: str
        def __init__(self, errorMessages: _Optional[_Union[_struct_pb2.ListValue, _Mapping]] = ..., addedBy: _Optional[str] = ...) -> None: ...
    class Processing(_message.Message):
        __slots__ = ("timeAdded", "addedBy", "timeStarted")
        TIMEADDED_FIELD_NUMBER: _ClassVar[int]
        ADDEDBY_FIELD_NUMBER: _ClassVar[int]
        TIMESTARTED_FIELD_NUMBER: _ClassVar[int]
        timeAdded: _timestamp_pb2.Timestamp
        addedBy: str
        timeStarted: _timestamp_pb2.Timestamp
        def __init__(self, timeAdded: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., addedBy: _Optional[str] = ..., timeStarted: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...) -> None: ...
    UNKNOWN_FIELD_NUMBER: _ClassVar[int]
    PENDING_FIELD_NUMBER: _ClassVar[int]
    RUNNING_FIELD_NUMBER: _ClassVar[int]
    COMPLETED_FIELD_NUMBER: _ClassVar[int]
    CANCELED_FIELD_NUMBER: _ClassVar[int]
    LOADING_FIELD_NUMBER: _ClassVar[int]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    PROCESSING_FIELD_NUMBER: _ClassVar[int]
    unknown: JobExecutionStatus.Unknown
    pending: JobExecutionStatus.Pending
    running: JobExecutionStatus.Running
    completed: JobExecutionStatus.Completed
    canceled: JobExecutionStatus.Canceled
    loading: JobExecutionStatus.Loading
    error: JobExecutionStatus.Error
    processing: JobExecutionStatus.Processing
    def __init__(self, unknown: _Optional[_Union[JobExecutionStatus.Unknown, _Mapping]] = ..., pending: _Optional[_Union[JobExecutionStatus.Pending, _Mapping]] = ..., running: _Optional[_Union[JobExecutionStatus.Running, _Mapping]] = ..., completed: _Optional[_Union[JobExecutionStatus.Completed, _Mapping]] = ..., canceled: _Optional[_Union[JobExecutionStatus.Canceled, _Mapping]] = ..., loading: _Optional[_Union[JobExecutionStatus.Loading, _Mapping]] = ..., error: _Optional[_Union[JobExecutionStatus.Error, _Mapping]] = ..., processing: _Optional[_Union[JobExecutionStatus.Processing, _Mapping]] = ...) -> None: ...

class GetPendingJobsResponse(_message.Message):
    __slots__ = ("pendingJobs",)
    class PendingJobsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: JobExecutionStatus.Pending
        def __init__(self, key: _Optional[str] = ..., value: _Optional[_Union[JobExecutionStatus.Pending, _Mapping]] = ...) -> None: ...
    PENDINGJOBS_FIELD_NUMBER: _ClassVar[int]
    pendingJobs: _containers.MessageMap[str, JobExecutionStatus.Pending]
    def __init__(self, pendingJobs: _Optional[_Mapping[str, JobExecutionStatus.Pending]] = ...) -> None: ...

class JobQueryParams(_message.Message):
    __slots__ = ("quantumMachineId", "jobId", "userId", "position")
    QUANTUMMACHINEID_FIELD_NUMBER: _ClassVar[int]
    JOBID_FIELD_NUMBER: _ClassVar[int]
    USERID_FIELD_NUMBER: _ClassVar[int]
    POSITION_FIELD_NUMBER: _ClassVar[int]
    quantumMachineId: str
    jobId: QueryValueMatcher
    userId: QueryValueMatcher
    position: _wrappers_pb2.UInt32Value
    def __init__(self, quantumMachineId: _Optional[str] = ..., jobId: _Optional[_Union[QueryValueMatcher, _Mapping]] = ..., userId: _Optional[_Union[QueryValueMatcher, _Mapping]] = ..., position: _Optional[_Union[_wrappers_pb2.UInt32Value, _Mapping]] = ...) -> None: ...

class QueryValueMatcher(_message.Message):
    __slots__ = ("any", "value")
    ANY_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    any: bool
    value: str
    def __init__(self, any: bool = ..., value: _Optional[str] = ...) -> None: ...

class RemovePendingJobsResponse(_message.Message):
    __slots__ = ("numbersOfJobsRemoved",)
    NUMBERSOFJOBSREMOVED_FIELD_NUMBER: _ClassVar[int]
    numbersOfJobsRemoved: int
    def __init__(self, numbersOfJobsRemoved: _Optional[int] = ...) -> None: ...

class SimulationRequest(_message.Message):
    __slots__ = ("config", "highLevelProgram", "simulate", "controllerConnections")
    CONFIG_FIELD_NUMBER: _ClassVar[int]
    HIGHLEVELPROGRAM_FIELD_NUMBER: _ClassVar[int]
    SIMULATE_FIELD_NUMBER: _ClassVar[int]
    CONTROLLERCONNECTIONS_FIELD_NUMBER: _ClassVar[int]
    config: _inc_qua_config_pb2.QuaConfig
    highLevelProgram: _inc_qua_pb2.QuaProgram
    simulate: ExecutionRequest.Simulate
    controllerConnections: _containers.RepeatedCompositeFieldContainer[InterOpxConnection]
    def __init__(self, config: _Optional[_Union[_inc_qua_config_pb2.QuaConfig, _Mapping]] = ..., highLevelProgram: _Optional[_Union[_inc_qua_pb2.QuaProgram, _Mapping]] = ..., simulate: _Optional[_Union[ExecutionRequest.Simulate, _Mapping]] = ..., controllerConnections: _Optional[_Iterable[_Union[InterOpxConnection, _Mapping]]] = ...) -> None: ...

class InterOpxAddress(_message.Message):
    __slots__ = ("controller", "left")
    CONTROLLER_FIELD_NUMBER: _ClassVar[int]
    LEFT_FIELD_NUMBER: _ClassVar[int]
    controller: str
    left: bool
    def __init__(self, controller: _Optional[str] = ..., left: bool = ...) -> None: ...

class InterOpxTarget(_message.Message):
    __slots__ = ("direct",)
    DIRECT_FIELD_NUMBER: _ClassVar[int]
    direct: InterOpxAddress
    def __init__(self, direct: _Optional[_Union[InterOpxAddress, _Mapping]] = ...) -> None: ...

class InterOpxChannel(_message.Message):
    __slots__ = ("controller", "channelNumber")
    CONTROLLER_FIELD_NUMBER: _ClassVar[int]
    CHANNELNUMBER_FIELD_NUMBER: _ClassVar[int]
    controller: str
    channelNumber: int
    def __init__(self, controller: _Optional[str] = ..., channelNumber: _Optional[int] = ...) -> None: ...

class InterOpxConnection(_message.Message):
    __slots__ = ("source", "target", "addressToAddress", "channelToChannel")
    class AddressToAddress(_message.Message):
        __slots__ = ("source", "target")
        SOURCE_FIELD_NUMBER: _ClassVar[int]
        TARGET_FIELD_NUMBER: _ClassVar[int]
        source: InterOpxAddress
        target: InterOpxAddress
        def __init__(self, source: _Optional[_Union[InterOpxAddress, _Mapping]] = ..., target: _Optional[_Union[InterOpxAddress, _Mapping]] = ...) -> None: ...
    class ChannelToChannel(_message.Message):
        __slots__ = ("source", "target")
        SOURCE_FIELD_NUMBER: _ClassVar[int]
        TARGET_FIELD_NUMBER: _ClassVar[int]
        source: InterOpxChannel
        target: InterOpxChannel
        def __init__(self, source: _Optional[_Union[InterOpxChannel, _Mapping]] = ..., target: _Optional[_Union[InterOpxChannel, _Mapping]] = ...) -> None: ...
    SOURCE_FIELD_NUMBER: _ClassVar[int]
    TARGET_FIELD_NUMBER: _ClassVar[int]
    ADDRESSTOADDRESS_FIELD_NUMBER: _ClassVar[int]
    CHANNELTOCHANNEL_FIELD_NUMBER: _ClassVar[int]
    source: InterOpxAddress
    target: InterOpxTarget
    addressToAddress: InterOpxConnection.AddressToAddress
    channelToChannel: InterOpxConnection.ChannelToChannel
    def __init__(self, source: _Optional[_Union[InterOpxAddress, _Mapping]] = ..., target: _Optional[_Union[InterOpxTarget, _Mapping]] = ..., addressToAddress: _Optional[_Union[InterOpxConnection.AddressToAddress, _Mapping]] = ..., channelToChannel: _Optional[_Union[InterOpxConnection.ChannelToChannel, _Mapping]] = ...) -> None: ...

class SimulationResponse(_message.Message):
    __slots__ = ("success", "jobId", "configValidationErrors", "physicalValidationErrors", "messages", "simulated")
    SUCCESS_FIELD_NUMBER: _ClassVar[int]
    JOBID_FIELD_NUMBER: _ClassVar[int]
    CONFIGVALIDATIONERRORS_FIELD_NUMBER: _ClassVar[int]
    PHYSICALVALIDATIONERRORS_FIELD_NUMBER: _ClassVar[int]
    MESSAGES_FIELD_NUMBER: _ClassVar[int]
    SIMULATED_FIELD_NUMBER: _ClassVar[int]
    success: bool
    jobId: str
    configValidationErrors: _containers.RepeatedCompositeFieldContainer[_qm_manager_pb2.ConfigValidationMessage]
    physicalValidationErrors: _containers.RepeatedCompositeFieldContainer[_qm_manager_pb2.PhysicalValidationMessage]
    messages: _containers.RepeatedCompositeFieldContainer[_compiler_pb2.CompilerMessage]
    simulated: SimulatedResponsePart
    def __init__(self, success: bool = ..., jobId: _Optional[str] = ..., configValidationErrors: _Optional[_Iterable[_Union[_qm_manager_pb2.ConfigValidationMessage, _Mapping]]] = ..., physicalValidationErrors: _Optional[_Iterable[_Union[_qm_manager_pb2.PhysicalValidationMessage, _Mapping]]] = ..., messages: _Optional[_Iterable[_Union[_compiler_pb2.CompilerMessage, _Mapping]]] = ..., simulated: _Optional[_Union[SimulatedResponsePart, _Mapping]] = ...) -> None: ...

class IsJobRunningRequest(_message.Message):
    __slots__ = ("jobId",)
    JOBID_FIELD_NUMBER: _ClassVar[int]
    jobId: str
    def __init__(self, jobId: _Optional[str] = ...) -> None: ...

class IsJobRunningResponse(_message.Message):
    __slots__ = ("jobId", "isRunning")
    JOBID_FIELD_NUMBER: _ClassVar[int]
    ISRUNNING_FIELD_NUMBER: _ClassVar[int]
    jobId: str
    isRunning: bool
    def __init__(self, jobId: _Optional[str] = ..., isRunning: bool = ...) -> None: ...

class IsJobAcquiringDataRequest(_message.Message):
    __slots__ = ("jobId",)
    JOBID_FIELD_NUMBER: _ClassVar[int]
    jobId: str
    def __init__(self, jobId: _Optional[str] = ...) -> None: ...

class IsJobAcquiringDataResponse(_message.Message):
    __slots__ = ("jobId", "acquiringStatus")
    class AcquiringStatus(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        ACQUIRE_STOPPED: _ClassVar[IsJobAcquiringDataResponse.AcquiringStatus]
        NO_DATA_TO_ACQUIRE: _ClassVar[IsJobAcquiringDataResponse.AcquiringStatus]
        HAS_DATA_TO_ACQUIRE: _ClassVar[IsJobAcquiringDataResponse.AcquiringStatus]
    ACQUIRE_STOPPED: IsJobAcquiringDataResponse.AcquiringStatus
    NO_DATA_TO_ACQUIRE: IsJobAcquiringDataResponse.AcquiringStatus
    HAS_DATA_TO_ACQUIRE: IsJobAcquiringDataResponse.AcquiringStatus
    JOBID_FIELD_NUMBER: _ClassVar[int]
    ACQUIRINGSTATUS_FIELD_NUMBER: _ClassVar[int]
    jobId: str
    acquiringStatus: IsJobAcquiringDataResponse.AcquiringStatus
    def __init__(self, jobId: _Optional[str] = ..., acquiringStatus: _Optional[_Union[IsJobAcquiringDataResponse.AcquiringStatus, str]] = ...) -> None: ...

class QmDataRequest(_message.Message):
    __slots__ = ("io_value_Request",)
    IO_VALUE_REQUEST_FIELD_NUMBER: _ClassVar[int]
    io_value_Request: _containers.RepeatedCompositeFieldContainer[IOValueRequest]
    def __init__(self, io_value_Request: _Optional[_Iterable[_Union[IOValueRequest, _Mapping]]] = ...) -> None: ...

class QmDataResponse(_message.Message):
    __slots__ = ("io_value_response", "success", "errors")
    class IOValueResponse(_message.Message):
        __slots__ = ("request", "values")
        REQUEST_FIELD_NUMBER: _ClassVar[int]
        VALUES_FIELD_NUMBER: _ClassVar[int]
        request: IOValueRequest
        values: _compiler_pb2.QuaValues
        def __init__(self, request: _Optional[_Union[IOValueRequest, _Mapping]] = ..., values: _Optional[_Union[_compiler_pb2.QuaValues, _Mapping]] = ...) -> None: ...
    IO_VALUE_RESPONSE_FIELD_NUMBER: _ClassVar[int]
    SUCCESS_FIELD_NUMBER: _ClassVar[int]
    ERRORS_FIELD_NUMBER: _ClassVar[int]
    io_value_response: _containers.RepeatedCompositeFieldContainer[QmDataResponse.IOValueResponse]
    success: bool
    errors: _containers.RepeatedCompositeFieldContainer[_general_messages_pb2.ErrorMessage]
    def __init__(self, io_value_response: _Optional[_Iterable[_Union[QmDataResponse.IOValueResponse, _Mapping]]] = ..., success: bool = ..., errors: _Optional[_Iterable[_Union[_general_messages_pb2.ErrorMessage, _Mapping]]] = ...) -> None: ...

class IOValueRequest(_message.Message):
    __slots__ = ("pulser_number", "io_number", "jobId", "quantumMachineId")
    PULSER_NUMBER_FIELD_NUMBER: _ClassVar[int]
    IO_NUMBER_FIELD_NUMBER: _ClassVar[int]
    JOBID_FIELD_NUMBER: _ClassVar[int]
    QUANTUMMACHINEID_FIELD_NUMBER: _ClassVar[int]
    pulser_number: int
    io_number: int
    jobId: str
    quantumMachineId: str
    def __init__(self, pulser_number: _Optional[int] = ..., io_number: _Optional[int] = ..., jobId: _Optional[str] = ..., quantumMachineId: _Optional[str] = ...) -> None: ...

class ResumeRequest(_message.Message):
    __slots__ = ("jobId", "pulsersBitmap")
    JOBID_FIELD_NUMBER: _ClassVar[int]
    PULSERSBITMAP_FIELD_NUMBER: _ClassVar[int]
    jobId: str
    pulsersBitmap: int
    def __init__(self, jobId: _Optional[str] = ..., pulsersBitmap: _Optional[int] = ...) -> None: ...

class ResumeResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class PausedStatusRequest(_message.Message):
    __slots__ = ("jobId",)
    JOBID_FIELD_NUMBER: _ClassVar[int]
    jobId: str
    def __init__(self, jobId: _Optional[str] = ...) -> None: ...

class PausedStatusResponse(_message.Message):
    __slots__ = ("pulsersBitmap", "ok", "isPaused")
    PULSERSBITMAP_FIELD_NUMBER: _ClassVar[int]
    OK_FIELD_NUMBER: _ClassVar[int]
    ISPAUSED_FIELD_NUMBER: _ClassVar[int]
    pulsersBitmap: int
    ok: bool
    isPaused: bool
    def __init__(self, pulsersBitmap: _Optional[int] = ..., ok: bool = ..., isPaused: bool = ...) -> None: ...

class PullResultRequest(_message.Message):
    __slots__ = ("jobId",)
    JOBID_FIELD_NUMBER: _ClassVar[int]
    jobId: str
    def __init__(self, jobId: _Optional[str] = ...) -> None: ...

class PullAnalysedResultsRequest(_message.Message):
    __slots__ = ("jobId", "metadata")
    JOBID_FIELD_NUMBER: _ClassVar[int]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    jobId: str
    metadata: str
    def __init__(self, jobId: _Optional[str] = ..., metadata: _Optional[str] = ...) -> None: ...

class PullResultResponse(_message.Message):
    __slots__ = ("data", "offset", "bytesLeft", "controllerName", "applicationNumber", "implementationName")
    DATA_FIELD_NUMBER: _ClassVar[int]
    OFFSET_FIELD_NUMBER: _ClassVar[int]
    BYTESLEFT_FIELD_NUMBER: _ClassVar[int]
    CONTROLLERNAME_FIELD_NUMBER: _ClassVar[int]
    APPLICATIONNUMBER_FIELD_NUMBER: _ClassVar[int]
    IMPLEMENTATIONNAME_FIELD_NUMBER: _ClassVar[int]
    data: bytes
    offset: int
    bytesLeft: int
    controllerName: str
    applicationNumber: int
    implementationName: str
    def __init__(self, data: _Optional[bytes] = ..., offset: _Optional[int] = ..., bytesLeft: _Optional[int] = ..., controllerName: _Optional[str] = ..., applicationNumber: _Optional[int] = ..., implementationName: _Optional[str] = ...) -> None: ...

class HealthCheckResponse(_message.Message):
    __slots__ = ("message", "ok", "errorMessages", "warningMessages")
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    OK_FIELD_NUMBER: _ClassVar[int]
    ERRORMESSAGES_FIELD_NUMBER: _ClassVar[int]
    WARNINGMESSAGES_FIELD_NUMBER: _ClassVar[int]
    message: _containers.RepeatedScalarFieldContainer[str]
    ok: bool
    errorMessages: _containers.RepeatedScalarFieldContainer[str]
    warningMessages: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, message: _Optional[_Iterable[str]] = ..., ok: bool = ..., errorMessages: _Optional[_Iterable[str]] = ..., warningMessages: _Optional[_Iterable[str]] = ...) -> None: ...

class ExecutionRequest(_message.Message):
    __slots__ = ("highLevelProgram", "lowLevelProgram", "streamDurationLimit", "streamDataLimit", "forceExecution", "simulate", "dryRun", "quantumMachineId")
    class Simulate(_message.Message):
        __slots__ = ("duration", "simulateAnalogOutputs", "simulateDigitalOutputs", "includeAnalogSamples", "includeDigitalSamples", "includeAnalogWaveforms", "includeDigitalWaveforms", "simulationInterface", "extraProcessingTimeoutMs")
        class SimulationInterface(_message.Message):
            __slots__ = ("none", "loopback", "raw")
            # class None(_message.Message):
#                 __slots__ = ()
#                 def __init__(self) -> None: ...
            class Loopback(_message.Message):
                __slots__ = ("connections", "latency", "noisePower")
                class Connections(_message.Message):
                    __slots__ = ("fromController", "fromFem", "fromPort", "toController", "toFem", "toPort")
                    FROMCONTROLLER_FIELD_NUMBER: _ClassVar[int]
                    FROMFEM_FIELD_NUMBER: _ClassVar[int]
                    FROMPORT_FIELD_NUMBER: _ClassVar[int]
                    TOCONTROLLER_FIELD_NUMBER: _ClassVar[int]
                    TOFEM_FIELD_NUMBER: _ClassVar[int]
                    TOPORT_FIELD_NUMBER: _ClassVar[int]
                    fromController: str
                    fromFem: int
                    fromPort: int
                    toController: str
                    toFem: int
                    toPort: int
                    def __init__(self, fromController: _Optional[str] = ..., fromFem: _Optional[int] = ..., fromPort: _Optional[int] = ..., toController: _Optional[str] = ..., toFem: _Optional[int] = ..., toPort: _Optional[int] = ...) -> None: ...
                CONNECTIONS_FIELD_NUMBER: _ClassVar[int]
                LATENCY_FIELD_NUMBER: _ClassVar[int]
                NOISEPOWER_FIELD_NUMBER: _ClassVar[int]
                connections: _containers.RepeatedCompositeFieldContainer[ExecutionRequest.Simulate.SimulationInterface.Loopback.Connections]
                latency: int
                noisePower: float
                def __init__(self, connections: _Optional[_Iterable[_Union[ExecutionRequest.Simulate.SimulationInterface.Loopback.Connections, _Mapping]]] = ..., latency: _Optional[int] = ..., noisePower: _Optional[float] = ...) -> None: ...
            class RawInterface(_message.Message):
                __slots__ = ("connections", "noisePower")
                class Connections(_message.Message):
                    __slots__ = ("fromController", "fromFem", "fromPort", "toSamples")
                    FROMCONTROLLER_FIELD_NUMBER: _ClassVar[int]
                    FROMFEM_FIELD_NUMBER: _ClassVar[int]
                    FROMPORT_FIELD_NUMBER: _ClassVar[int]
                    TOSAMPLES_FIELD_NUMBER: _ClassVar[int]
                    fromController: str
                    fromFem: int
                    fromPort: int
                    toSamples: _containers.RepeatedScalarFieldContainer[float]
                    def __init__(self, fromController: _Optional[str] = ..., fromFem: _Optional[int] = ..., fromPort: _Optional[int] = ..., toSamples: _Optional[_Iterable[float]] = ...) -> None: ...
                CONNECTIONS_FIELD_NUMBER: _ClassVar[int]
                NOISEPOWER_FIELD_NUMBER: _ClassVar[int]
                connections: _containers.RepeatedCompositeFieldContainer[ExecutionRequest.Simulate.SimulationInterface.RawInterface.Connections]
                noisePower: float
                def __init__(self, connections: _Optional[_Iterable[_Union[ExecutionRequest.Simulate.SimulationInterface.RawInterface.Connections, _Mapping]]] = ..., noisePower: _Optional[float] = ...) -> None: ...
            NONE_FIELD_NUMBER: _ClassVar[int]
            LOOPBACK_FIELD_NUMBER: _ClassVar[int]
            RAW_FIELD_NUMBER: _ClassVar[int]
            none: getattr(ExecutionRequest.Simulate.SimulationInterface, 'None')
            loopback: ExecutionRequest.Simulate.SimulationInterface.Loopback
            raw: ExecutionRequest.Simulate.SimulationInterface.RawInterface
            def __init__(self, none: _Optional[_Union[getattr(ExecutionRequest.Simulate.SimulationInterface, 'None'), _Mapping]] = ..., loopback: _Optional[_Union[ExecutionRequest.Simulate.SimulationInterface.Loopback, _Mapping]] = ..., raw: _Optional[_Union[ExecutionRequest.Simulate.SimulationInterface.RawInterface, _Mapping]] = ...) -> None: ...
        DURATION_FIELD_NUMBER: _ClassVar[int]
        SIMULATEANALOGOUTPUTS_FIELD_NUMBER: _ClassVar[int]
        SIMULATEDIGITALOUTPUTS_FIELD_NUMBER: _ClassVar[int]
        INCLUDEANALOGSAMPLES_FIELD_NUMBER: _ClassVar[int]
        INCLUDEDIGITALSAMPLES_FIELD_NUMBER: _ClassVar[int]
        INCLUDEANALOGWAVEFORMS_FIELD_NUMBER: _ClassVar[int]
        INCLUDEDIGITALWAVEFORMS_FIELD_NUMBER: _ClassVar[int]
        SIMULATIONINTERFACE_FIELD_NUMBER: _ClassVar[int]
        EXTRAPROCESSINGTIMEOUTMS_FIELD_NUMBER: _ClassVar[int]
        duration: int
        simulateAnalogOutputs: bool
        simulateDigitalOutputs: bool
        includeAnalogSamples: bool
        includeDigitalSamples: bool
        includeAnalogWaveforms: bool
        includeDigitalWaveforms: bool
        simulationInterface: ExecutionRequest.Simulate.SimulationInterface
        extraProcessingTimeoutMs: int
        def __init__(self, duration: _Optional[int] = ..., simulateAnalogOutputs: bool = ..., simulateDigitalOutputs: bool = ..., includeAnalogSamples: bool = ..., includeDigitalSamples: bool = ..., includeAnalogWaveforms: bool = ..., includeDigitalWaveforms: bool = ..., simulationInterface: _Optional[_Union[ExecutionRequest.Simulate.SimulationInterface, _Mapping]] = ..., extraProcessingTimeoutMs: _Optional[int] = ...) -> None: ...
    HIGHLEVELPROGRAM_FIELD_NUMBER: _ClassVar[int]
    LOWLEVELPROGRAM_FIELD_NUMBER: _ClassVar[int]
    STREAMDURATIONLIMIT_FIELD_NUMBER: _ClassVar[int]
    STREAMDATALIMIT_FIELD_NUMBER: _ClassVar[int]
    FORCEEXECUTION_FIELD_NUMBER: _ClassVar[int]
    SIMULATE_FIELD_NUMBER: _ClassVar[int]
    DRYRUN_FIELD_NUMBER: _ClassVar[int]
    QUANTUMMACHINEID_FIELD_NUMBER: _ClassVar[int]
    highLevelProgram: _inc_qua_pb2.QuaProgram
    lowLevelProgram: bytes
    streamDurationLimit: int
    streamDataLimit: int
    forceExecution: bool
    simulate: ExecutionRequest.Simulate
    dryRun: bool
    quantumMachineId: str
    def __init__(self, highLevelProgram: _Optional[_Union[_inc_qua_pb2.QuaProgram, _Mapping]] = ..., lowLevelProgram: _Optional[bytes] = ..., streamDurationLimit: _Optional[int] = ..., streamDataLimit: _Optional[int] = ..., forceExecution: bool = ..., simulate: _Optional[_Union[ExecutionRequest.Simulate, _Mapping]] = ..., dryRun: bool = ..., quantumMachineId: _Optional[str] = ...) -> None: ...

class ExecutionResponse(_message.Message):
    __slots__ = ("ok", "jobId", "messages", "metadata", "simulated", "config")
    OK_FIELD_NUMBER: _ClassVar[int]
    JOBID_FIELD_NUMBER: _ClassVar[int]
    MESSAGES_FIELD_NUMBER: _ClassVar[int]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    SIMULATED_FIELD_NUMBER: _ClassVar[int]
    CONFIG_FIELD_NUMBER: _ClassVar[int]
    ok: bool
    jobId: str
    messages: _containers.RepeatedCompositeFieldContainer[_compiler_pb2.CompilerMessage]
    metadata: str
    simulated: SimulatedResponsePart
    config: _inc_qua_config_pb2.QuaConfig
    def __init__(self, ok: bool = ..., jobId: _Optional[str] = ..., messages: _Optional[_Iterable[_Union[_compiler_pb2.CompilerMessage, _Mapping]]] = ..., metadata: _Optional[str] = ..., simulated: _Optional[_Union[SimulatedResponsePart, _Mapping]] = ..., config: _Optional[_Union[_inc_qua_config_pb2.QuaConfig, _Mapping]] = ...) -> None: ...

class SimulatedResponsePart(_message.Message):
    __slots__ = ("analogOutputs", "digitalOutputs", "waveformReport", "errors")
    ANALOGOUTPUTS_FIELD_NUMBER: _ClassVar[int]
    DIGITALOUTPUTS_FIELD_NUMBER: _ClassVar[int]
    WAVEFORMREPORT_FIELD_NUMBER: _ClassVar[int]
    ERRORS_FIELD_NUMBER: _ClassVar[int]
    analogOutputs: _struct_pb2.Struct
    digitalOutputs: _struct_pb2.Struct
    waveformReport: _struct_pb2.Struct
    errors: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, analogOutputs: _Optional[_Union[_struct_pb2.Struct, _Mapping]] = ..., digitalOutputs: _Optional[_Union[_struct_pb2.Struct, _Mapping]] = ..., waveformReport: _Optional[_Union[_struct_pb2.Struct, _Mapping]] = ..., errors: _Optional[_Iterable[str]] = ...) -> None: ...

class ResetDataProcessingRequest(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class ResetDataProcessingResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class HaltRequest(_message.Message):
    __slots__ = ("jobId",)
    JOBID_FIELD_NUMBER: _ClassVar[int]
    jobId: str
    def __init__(self, jobId: _Optional[str] = ...) -> None: ...

class HaltResponse(_message.Message):
    __slots__ = ("ok",)
    OK_FIELD_NUMBER: _ClassVar[int]
    ok: bool
    def __init__(self, ok: bool = ...) -> None: ...

class PeekRequest(_message.Message):
    __slots__ = ("address", "controllerId")
    ADDRESS_FIELD_NUMBER: _ClassVar[int]
    CONTROLLERID_FIELD_NUMBER: _ClassVar[int]
    address: int
    controllerId: str
    def __init__(self, address: _Optional[int] = ..., controllerId: _Optional[str] = ...) -> None: ...

class PeekResponse(_message.Message):
    __slots__ = ("value",)
    VALUE_FIELD_NUMBER: _ClassVar[int]
    value: int
    def __init__(self, value: _Optional[int] = ...) -> None: ...

class PokeRequest(_message.Message):
    __slots__ = ("address", "value", "controllerId")
    ADDRESS_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    CONTROLLERID_FIELD_NUMBER: _ClassVar[int]
    address: int
    value: int
    controllerId: str
    def __init__(self, address: _Optional[int] = ..., value: _Optional[int] = ..., controllerId: _Optional[str] = ...) -> None: ...

class PokeResponse(_message.Message):
    __slots__ = ("ok",)
    OK_FIELD_NUMBER: _ClassVar[int]
    ok: bool
    def __init__(self, ok: bool = ...) -> None: ...

class GetSimulatedQuantumStateRequest(_message.Message):
    __slots__ = ("jobId",)
    JOBID_FIELD_NUMBER: _ClassVar[int]
    jobId: str
    def __init__(self, jobId: _Optional[str] = ...) -> None: ...

class GetSimulatedQuantumStateResponse(_message.Message):
    __slots__ = ("jobId", "ok", "state")
    JOBID_FIELD_NUMBER: _ClassVar[int]
    OK_FIELD_NUMBER: _ClassVar[int]
    STATE_FIELD_NUMBER: _ClassVar[int]
    jobId: str
    ok: bool
    state: DensityMatrix
    def __init__(self, jobId: _Optional[str] = ..., ok: bool = ..., state: _Optional[_Union[DensityMatrix, _Mapping]] = ...) -> None: ...

class DensityMatrix(_message.Message):
    __slots__ = ("timeStamp", "data")
    TIMESTAMP_FIELD_NUMBER: _ClassVar[int]
    DATA_FIELD_NUMBER: _ClassVar[int]
    timeStamp: int
    data: _containers.RepeatedCompositeFieldContainer[ComplexNumber]
    def __init__(self, timeStamp: _Optional[int] = ..., data: _Optional[_Iterable[_Union[ComplexNumber, _Mapping]]] = ...) -> None: ...

class ComplexNumber(_message.Message):
    __slots__ = ("re", "im")
    RE_FIELD_NUMBER: _ClassVar[int]
    IM_FIELD_NUMBER: _ClassVar[int]
    re: float
    im: float
    def __init__(self, re: _Optional[float] = ..., im: _Optional[float] = ...) -> None: ...

class PerformHalDebugCommandRequest(_message.Message):
    __slots__ = ("controllerName", "command")
    CONTROLLERNAME_FIELD_NUMBER: _ClassVar[int]
    COMMAND_FIELD_NUMBER: _ClassVar[int]
    controllerName: str
    command: str
    def __init__(self, controllerName: _Optional[str] = ..., command: _Optional[str] = ...) -> None: ...

class PerformHalDebugCommandResponse(_message.Message):
    __slots__ = ("success", "response")
    SUCCESS_FIELD_NUMBER: _ClassVar[int]
    RESPONSE_FIELD_NUMBER: _ClassVar[int]
    success: bool
    response: str
    def __init__(self, success: bool = ..., response: _Optional[str] = ...) -> None: ...



from qm.grpc.octave.v1 import api_pb2


class _BuilderList:
    def __init__(self) -> None:
        super().__init__()
        self._items: dict[int, api_pb2.SingleUpdate] = {}

    def get_items(self) -> list[api_pb2.SingleUpdate]:
        return list(self._items.values())


class _SynthBuilderList(_BuilderList):
    def __getitem__(self, item: int) -> api_pb2.SynthUpdate:
        if item not in self._items:
            self._items[item] = api_pb2.SingleUpdate(synth=api_pb2.SynthUpdate(index=item))
        return self._items[item].synth


class _DownBuilderList(_BuilderList):
    def __getitem__(self, item: int) -> api_pb2.RFDownConvUpdate:
        if item not in self._items:
            self._items[item] = api_pb2.SingleUpdate(rf_down_conv=api_pb2.RFDownConvUpdate(index=item))
        return self._items[item].rf_down_conv


class _UpBuilderList(_BuilderList):
    def __getitem__(self, item: int) -> api_pb2.RFUpConvUpdate:
        if item not in self._items:
            self._items[item] = api_pb2.SingleUpdate(rf_up_conv=api_pb2.RFUpConvUpdate(index=item))
        return self._items[item].rf_up_conv


class _IFBuilderList(_BuilderList):
    def __getitem__(self, item: int) -> api_pb2.IFDownConvUpdate:
        if item not in self._items:
            self._items[item] = api_pb2.SingleUpdate(if_down_conv=api_pb2.IFDownConvUpdate(index=item))
        return self._items[item].if_down_conv


class _ClockBuilderList(_BuilderList):
    def __getitem__(self, item: int) -> api_pb2.ClockUpdate:
        if item not in self._items:
            self._items[item] = api_pb2.SingleUpdate(clock=api_pb2.ClockUpdate())
        return self._items[item].clock_dist  # type: ignore  # (YR) - seems like a bug


class ClientRequestBuilder:
    def __init__(self) -> None:
        super().__init__()
        self._synth_builder_list = _SynthBuilderList()
        self._up_builder_list = _UpBuilderList()
        self._down_builder_list = _DownBuilderList()
        self._if_builder_list = _IFBuilderList()
        self._clock_builder_list = _ClockBuilderList()
        self._updates: list[api_pb2.SingleUpdate] = []

    @property
    def synth(self) -> _SynthBuilderList:
        return self._synth_builder_list

    @property
    def up(self) -> _UpBuilderList:
        return self._up_builder_list

    @property
    def down(self) -> _DownBuilderList:
        return self._down_builder_list

    @property
    def ifconv(self) -> _IFBuilderList:
        return self._if_builder_list

    @property
    def clk(self) -> _ClockBuilderList:
        return self._clock_builder_list

    def get_updates(self) -> list[api_pb2.SingleUpdate]:
        return (
            self._synth_builder_list.get_items()
            + self._down_builder_list.get_items()
            + self._up_builder_list.get_items()
            + self._if_builder_list.get_items()
            + self._clock_builder_list.get_items()
        )
